-- 
-- emsx_top.vhd
--   ESE MSX-SYSTEM3 / MSX clone on a Cyclone FPGA (ALTERA)
--   Revision 1.00
--
-- modified for Altera DE1 by caro 2007..2008
-- 
-- Copyright (c) 2006 Kazuhiro Tsujikawa (ESE Artists' factory)
-- All rights reserved.
-- 
-- Redistribution and use of this source code or any derivative works, are 
-- permitted provided that the following conditions are met:
--
-- 1. Redistributions of source code must retain the above copyright notice, 
--    this list of conditions and the following disclaimer.
-- 2. Redistributions in binary form must reproduce the above copyright 
--    notice, this list of conditions and the following disclaimer in the 
--    documentation and/or other materials provided with the distribution.
-- 3. Redistributions may not be sold, nor may they be used in a commercial 
--    product or activity without specific prior written permission.
--
-- THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
-- "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED 
-- TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR 
-- PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR 
-- CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
-- EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
-- PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
-- OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
-- WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR 
-- OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF 
-- ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
-- 

LIBRARY IEEE;
	USE IEEE.STD_LOGIC_1164.ALL;
	USE IEEE.STD_LOGIC_UNSIGNED.ALL;
	USE WORK.VDP_PACKAGE.ALL;

ENTITY EMSX_TOP IS
	PORT(
		-- CLOCK, RESET PORTS
		CLOCK_50	: IN	STD_LOGIC;							-- INPUT CLOCK DE1 50 MHZ
		CLOCK_24	: IN	STD_LOGIC;							-- INPUT CLOCK DE1 24 MHZ
		PCLK24M		: IN	STD_LOGIC;							-- INPUT CLOCK DE1 24 MHZ
		CLOCK_27	: IN	STD_LOGIC;							-- INPUT CLOCK DE1 27 MHZ
		PCLK27M		: IN	STD_LOGIC;							-- INPUT CLOCK DE1 27 MHZ
		PEXTCLK		: IN	STD_LOGIC;							-- RESERVED (FOR MULTI FPGAS)
		PCPUCLK		: OUT	STD_LOGIC;							-- CPU CLOCK ... 3.58MHZ (UP TO 10.74MHZ/21.48MHZ)

		-- MSX CARTRIDGE SLOT PORTS
		PSLTRST_N	: IN	STD_LOGIC;							-- PCPURST_N RETURNS HERE
		PSLTSLTSL_N : INOUT	STD_LOGIC;
		PSLTSLTS2_N : INOUT	STD_LOGIC;
		PSLTIORQ_N	: INOUT	STD_LOGIC;
		PSLTRD_N	: INOUT	STD_LOGIC;
		PSLTWR_N	: INOUT	STD_LOGIC;
		PSLTADR		: INOUT	STD_LOGIC_VECTOR( 15 DOWNTO 0 );
		PSLTDAT		: INOUT	STD_LOGIC_VECTOR(  7 DOWNTO 0 );
		PSLTBDIR_N	: OUT	STD_LOGIC;							-- BUS DIRECTION (NOT USED IN	MASTER MODE)

		PSLTCS1_N	: INOUT	STD_LOGIC;
		PSLTCS2_N	: INOUT	STD_LOGIC;
		PSLTCS12_N	: INOUT	STD_LOGIC;
		PSLTRFSH_N	: INOUT	STD_LOGIC;
		PSLTWAIT_N	: INOUT	STD_LOGIC;
		PSLTINT_N	: INOUT	STD_LOGIC;
		PSLTM1_N	: INOUT	STD_LOGIC;
		PSLTMERQ_N	: INOUT	STD_LOGIC;

		PSLTRSV5	: OUT	STD_LOGIC;							-- RESERVED
		PSLTRSV16	: OUT	STD_LOGIC;							-- RESERVED (W/ EXTERNAL PULL-UP)
		PSLTSW1		: INOUT	STD_LOGIC;							-- RESERVED (W/ EXTERNAL PULL-UP)
		PSLTSW2		: INOUT	STD_LOGIC;							-- RESERVED

		-- SD-RAM PORTS
		PMEMCLK		: OUT	STD_LOGIC;							-- SD-RAM CLOCK
		PMEMCKE		: OUT	STD_LOGIC;							-- SD-RAM CLOCK ENABLE
		PMEMCS_N	: OUT	STD_LOGIC;							-- SD-RAM CHIP SELECT
		PMEMRAS_N	: OUT	STD_LOGIC;							-- SD-RAM ROW/RAS
		PMEMCAS_N	: OUT	STD_LOGIC;							-- SD-RAM /CAS
		PMEMWE_N	: OUT	STD_LOGIC;							-- SD-RAM /WE
		PMEMUDQ		: OUT	STD_LOGIC;							-- SD-RAM UDQM
		PMEMLDQ		: OUT	STD_LOGIC;							-- SD-RAM LDQM
		PMEMBA1		: OUT	STD_LOGIC;							-- SD-RAM BANK SELECT ADDRESS 1
		PMEMBA0		: OUT	STD_LOGIC;							-- SD-RAM BANK SELECT ADDRESS 0
		PMEMADR		: OUT	STD_LOGIC_VECTOR( 11 DOWNTO 0 );	-- SD-RAM ADDRESS
		PMEMDAT		: INOUT	STD_LOGIC_VECTOR( 15 DOWNTO 0 );	-- SD-RAM DATA

		-- PS/2 KEYBOARD PORTS
		PPS2CLK		: INOUT	STD_LOGIC;
		PPS2DAT		: INOUT	STD_LOGIC;

		-- JOYSTICK PORTS (PORT_A, PORT_B)
		PJOYA		: INOUT	STD_LOGIC_VECTOR( 5 DOWNTO 0);
		PSTRA		: OUT	STD_LOGIC;
		PJOYB		: INOUT	STD_LOGIC_VECTOR( 5 DOWNTO 0);
		PSTRB		: OUT	STD_LOGIC;

		-- SD/MMC SLOT PORTS
		PSD_CK		: OUT	STD_LOGIC;							-- PIN	5
		PSD_CM		: OUT	STD_LOGIC;							-- PIN	2
		PSD_DT3		: INOUT	STD_LOGIC;							-- PIN	1
		PSD_DT0		: INOUT	STD_LOGIC;							-- PIN	7

		-- DIP SWITCH, LAMP PORTS
		PSW			: IN	STD_LOGIC_VECTOR(  3 DOWNTO 0 );	-- 0 - PRESS; 1 - UNPRESS
		PDIP		: IN	STD_LOGIC_VECTOR(  9 DOWNTO 0 );	-- 0=ON,	1=OFF(DEFAULT ON SHIPMENT)
		PLEDG		: OUT	STD_LOGIC_VECTOR(  7 DOWNTO 0 );	-- 0=OFF, 1=ON(GREEN)
		PLEDR		: OUT	STD_LOGIC_VECTOR(  9 DOWNTO 0 );	-- 0=OFF, 1=ON(RED) ...POWER & SD/MMC ACCESS LAMP

		-- VIDEO, AUDIO/CMT PORTS
		PDAC_VR		: INOUT	STD_LOGIC_VECTOR(  5 DOWNTO 0 );	-- RGB_RED / SVIDEO_C
		PDAC_VG		: INOUT	STD_LOGIC_VECTOR(  5 DOWNTO 0 );	-- RGB_GRN / SVIDEO_Y
		PDAC_VB		: INOUT	STD_LOGIC_VECTOR(  5 DOWNTO 0 );	-- RGB_BLU / COMPOSITEVIDEO
		PDAC_S		: OUT	STD_LOGIC;							-- SOUND
		PCMT_OUT	: OUT	STD_LOGIC;							-- CMT OUTPUT
		PCMT_IN		: IN	STD_LOGIC;							-- CMT INPUT

		PVIDEOHS_N	: OUT	STD_LOGIC;							-- CSYNC(RGB15K), HSYNC(VGA31K)
		PVIDEOVS_N	: OUT	STD_LOGIC;							-- AUDIO(RGB15K), VSYNC(VGA31K)

		-- DE1 SRAM
		SRAM_DQ		: INOUT	STD_LOGIC_VECTOR( 15 DOWNTO 0 );
		SRAM_ADDR	: OUT	STD_LOGIC_VECTOR( 17 DOWNTO 0 );
		SRAM_UB_N	: OUT	STD_LOGIC;
		SRAM_LB_N	: OUT	STD_LOGIC;
		SRAM_WE_N	: OUT	STD_LOGIC;
		SRAM_CE_N	: OUT	STD_LOGIC;
		SRAM_OE_N	: OUT	STD_LOGIC;

		-- DE1 FLASH
		FL_DQ		: INOUT	STD_LOGIC_VECTOR(  7 DOWNTO 0 );
		FL_ADDR		: OUT	STD_LOGIC_VECTOR( 21 DOWNTO 0 );
		FL_RST_N	: OUT	STD_LOGIC;
		FL_WE_N		: OUT	STD_LOGIC;
		FL_OE_N		: OUT	STD_LOGIC;

		-- DE1 7-SEG DISPLAY
		HEX0		: OUT	STD_LOGIC_VECTOR(  6 DOWNTO 0 );
		HEX1		: OUT	STD_LOGIC_VECTOR(  6 DOWNTO 0 );
		HEX2		: OUT	STD_LOGIC_VECTOR(  6 DOWNTO 0 );
		HEX3		: OUT	STD_LOGIC_VECTOR(  6 DOWNTO 0 );

		-- DE1 I2C
		I2C_SCLK	: OUT	STD_LOGIC;
		I2C_SDAT	: INOUT	STD_LOGIC;

		-- DE1 AUDIO CODEC
		AUD_ADCLRCK	: OUT	STD_LOGIC;
		AUD_ADCDAT	: OUT	STD_LOGIC;
		AUD_XCK		: OUT	STD_LOGIC;
		AUD_DACLRCK	: OUT	STD_LOGIC;
		AUD_DACDAT	: OUT	STD_LOGIC;
		AUD_BCLK	: OUT	STD_LOGIC
	);
END EMSX_TOP;

ARCHITECTURE RTL OF EMSX_TOP IS

	--------------------------------------------------------------------------
	-- COMPONENT DEFINITION (PUBLIC MSX COMPONENT)
	--------------------------------------------------------------------------
	-- CPU
	COMPONENT T80A
		PORT(
			RESET_N		: IN	STD_LOGIC;
			CLK_N		: IN	STD_LOGIC;
			WAIT_N		: IN	STD_LOGIC;
			INT_N		: IN	STD_LOGIC;
			NMI_N		: IN	STD_LOGIC;
			BUSRQ_N		: IN	STD_LOGIC;
			M1_N		: OUT	STD_LOGIC;
			MREQ_N		: OUT	STD_LOGIC;
			IORQ_N		: OUT	STD_LOGIC;
			RD_N		: OUT	STD_LOGIC;
			WR_N		: OUT	STD_LOGIC;
			RFSH_N		: OUT	STD_LOGIC;
			HALT_N		: OUT	STD_LOGIC;
			BUSAK_N		: OUT	STD_LOGIC;
			A			: OUT	STD_LOGIC_VECTOR( 15 DOWNTO 0 );
			D			: INOUT	STD_LOGIC_VECTOR(  7 DOWNTO 0 )
		);
	END COMPONENT;

	-- BOOT LOADER ROM (INITIAL PROGRAM LOADER)
	COMPONENT IPLROM
		PORT(
			CLK			: IN	STD_LOGIC;
			ADR			: IN	STD_LOGIC_VECTOR( 15 DOWNTO 0 );
			DBI			: OUT	STD_LOGIC_VECTOR(  7 DOWNTO 0 )
		);
	END COMPONENT;

	-- MEGA-SD (SD/MMC CONTROLLER)
	COMPONENT MEGASD
		PORT(
			CLK21M		: IN	STD_LOGIC;
			RESET		: IN	STD_LOGIC;
			CLKENA		: IN	STD_LOGIC;
			REQ			: IN	STD_LOGIC;
			ACK			: OUT	STD_LOGIC;
			WRT			: IN	STD_LOGIC;
			ADR			: IN	STD_LOGIC_VECTOR( 15 DOWNTO 0 );
			DBI			: OUT	STD_LOGIC_VECTOR(  7 DOWNTO 0 );
			DBO			: IN	STD_LOGIC_VECTOR(  7 DOWNTO 0 );

			RAMREQ		: OUT	STD_LOGIC;
			RAMWRT		: OUT	STD_LOGIC;
			RAMADR		: OUT	STD_LOGIC_VECTOR( 19 DOWNTO 0 );
			RAMDBI		: IN	STD_LOGIC_VECTOR(  7 DOWNTO 0 );
			RAMDBO		: OUT	STD_LOGIC_VECTOR(  7 DOWNTO 0 );

			MMCDBI		: OUT	STD_LOGIC_VECTOR(  7 DOWNTO 0 );
			MMCENA		: OUT	STD_LOGIC;
			MMCACT		: OUT	STD_LOGIC;

			MMC_CK		: OUT	STD_LOGIC;
			MMC_CS		: OUT	STD_LOGIC;
			MMC_DI		: OUT	STD_LOGIC;
			MMC_DO		: IN	STD_LOGIC;

			EPC_CK		: OUT	STD_LOGIC;
			EPC_CS		: OUT	STD_LOGIC;
			EPC_OE		: OUT	STD_LOGIC;
			EPC_DI		: OUT	STD_LOGIC;
			EPC_DO		: IN	STD_LOGIC
		);
	END COMPONENT;

	-- MAPPER RAM
	COMPONENT MAPPER
		PORT(
			CLK21M		: IN	STD_LOGIC;
			RESET		: IN	STD_LOGIC;
			CLKENA		: IN	STD_LOGIC;
			REQ			: IN	STD_LOGIC;
			ACK			: OUT	STD_LOGIC;
			MEM			: IN	STD_LOGIC;
			WRT			: IN	STD_LOGIC;
			ADR			: IN	STD_LOGIC_VECTOR( 15 DOWNTO 0 );
			DBI			: OUT	STD_LOGIC_VECTOR(  7 DOWNTO 0 );
			DBO			: IN	STD_LOGIC_VECTOR(  7 DOWNTO 0 );

			RAMREQ		: OUT	STD_LOGIC;
			RAMWRT		: OUT	STD_LOGIC;
			RAMADR		: OUT	STD_LOGIC_VECTOR( 21 DOWNTO 0 );
			RAMDBI		: IN	STD_LOGIC_VECTOR(  7 DOWNTO 0 );
			RAMDBO		: OUT	STD_LOGIC_VECTOR(  7 DOWNTO 0 )
		);
	END COMPONENT;

	-- KEYBOARD CONTROLLER
	COMPONENT ESEPS2 IS
	PORT (
		CLK21M			: IN	STD_LOGIC;
		RESET			: IN	STD_LOGIC;
		CLKENA			: IN	STD_LOGIC;

		KMAP			: IN	STD_LOGIC;
		
		CAPS			: IN	STD_LOGIC;
		KANA			: IN	STD_LOGIC;
		PAUS			: OUT	STD_LOGIC;
		RESO			: OUT	STD_LOGIC;
		
		FKEYS			: OUT	STD_LOGIC_VECTOR(  7 DOWNTO 0 );

		PPS2CLK			: INOUT	STD_LOGIC;
		PPS2DAT			: INOUT	STD_LOGIC;
		PPIPORTC		: INOUT	STD_LOGIC_VECTOR(  7 DOWNTO 0 );
		PKEYX			: INOUT	STD_LOGIC_VECTOR(  7 DOWNTO 0 )
	);
	END COMPONENT;

	-- CLOCK-IC
	COMPONENT RTC
		PORT(
			CLK21M		: IN	STD_LOGIC;
			RESET		: IN	STD_LOGIC;
			CLKENA		: IN	STD_LOGIC;
			REQ			: IN	STD_LOGIC;
			ACK			: OUT	STD_LOGIC;
			WRT			: IN	STD_LOGIC;
			ADR			: IN	STD_LOGIC_VECTOR( 15 DOWNTO 0 );
			DBI			: OUT	STD_LOGIC_VECTOR(  7 DOWNTO 0 );
			DBO			: IN	STD_LOGIC_VECTOR(  7 DOWNTO 0 )
		);
	END COMPONENT;

	-- KANJI-ROM CONTROLLER
	COMPONENT KANJI IS
	PORT (
		CLK21M			: IN	STD_LOGIC;
		RESET			: IN	STD_LOGIC;
		CLKENA			: IN	STD_LOGIC;
		REQ				: IN	STD_LOGIC;
		ACK				: OUT	STD_LOGIC;
		WRT				: IN	STD_LOGIC;
		ADR				: IN	STD_LOGIC_VECTOR( 15 DOWNTO 0 );
		DBI				: OUT	STD_LOGIC_VECTOR(  7 DOWNTO 0 );
		DBO				: IN	STD_LOGIC_VECTOR(  7 DOWNTO 0 );

		RAMREQ			: OUT	STD_LOGIC;
		RAMADR			: OUT	STD_LOGIC_VECTOR( 17 DOWNTO 0 );
		RAMDBI			: IN	STD_LOGIC_VECTOR(  7 DOWNTO 0 );
		RAMDBO			: OUT	STD_LOGIC_VECTOR(  7 DOWNTO 0 )
	);
	END COMPONENT;

	-- V9958
	COMPONENT VDP
	PORT(
		-- VDP CLOCK ... 21.477MHZ
		CLK21M			: IN	STD_LOGIC;
		RESET			: IN	STD_LOGIC;
		REQ				: IN	STD_LOGIC;
		ACK				: OUT	STD_LOGIC;
		WRT				: IN	STD_LOGIC;
		ADR				: IN	STD_LOGIC_VECTOR( 15 DOWNTO 0 );
		DBI				: OUT	STD_LOGIC_VECTOR(  7 DOWNTO 0 );
		DBO				: IN	STD_LOGIC_VECTOR(  7 DOWNTO 0 );

		INT_N			: OUT	STD_LOGIC;

		PRAMOE_N		: OUT	STD_LOGIC;
		PRAMWE_N		: OUT	STD_LOGIC;
		PRAMADR			: OUT	STD_LOGIC_VECTOR( 16 DOWNTO 0 );
		PRAMDBI			: IN	STD_LOGIC_VECTOR( 15 DOWNTO 0 );
		PRAMDBO			: OUT	STD_LOGIC_VECTOR(  7 DOWNTO 0 );

		HISPEED_MODE	: IN	STD_LOGIC;

		-- VIDEO OUTPUT
		PVIDEOR			: OUT	STD_LOGIC_VECTOR(  5 DOWNTO 0 );
		PVIDEOG			: OUT	STD_LOGIC_VECTOR(  5 DOWNTO 0 );
		PVIDEOB			: OUT	STD_LOGIC_VECTOR(  5 DOWNTO 0 );

		PVIDEOHS_N		: OUT	STD_LOGIC;
		PVIDEOVS_N		: OUT	STD_LOGIC;
		PVIDEOCS_N		: OUT	STD_LOGIC;

		PVIDEODHCLK		: OUT	STD_LOGIC;
		PVIDEODLCLK		: OUT	STD_LOGIC;

		-- DISPLAY RESOLUTION (0=15KHZ, 1=31KHZ)
		DISPRESO		: IN	 STD_LOGIC
	);
	END COMPONENT;

	-- VIDEO ENCODER
	COMPONENT VENCODE
		PORT(
			CLK21M		: IN	STD_LOGIC;
			RESET		: IN	STD_LOGIC;
			VIDEOR		: IN	STD_LOGIC_VECTOR(  5 DOWNTO 0 );
			VIDEOG		: IN	STD_LOGIC_VECTOR(  5 DOWNTO 0 );
			VIDEOB		: IN	STD_LOGIC_VECTOR(  5 DOWNTO 0 );
			VIDEOHS_N 	: IN	STD_LOGIC;
			VIDEOVS_N 	: IN	STD_LOGIC;
			VIDEOY		: OUT	STD_LOGIC_VECTOR(  5 DOWNTO 0 );
			VIDEOC		: OUT	STD_LOGIC_VECTOR(  5 DOWNTO 0 );
			VIDEOV		: OUT	STD_LOGIC_VECTOR(  5 DOWNTO 0 )
		);
	END COMPONENT;

	-- PSG
	COMPONENT PSG
		PORT(
			CLK21M		: IN	STD_LOGIC;
			RESET		: IN	STD_LOGIC;
			CLKENA		: IN	STD_LOGIC;
			REQ			: IN	STD_LOGIC;
			ACK			: OUT	STD_LOGIC;
			WRT			: IN	STD_LOGIC;
			ADR			: IN	STD_LOGIC_VECTOR( 15 DOWNTO 0 );
			DBI			: OUT	STD_LOGIC_VECTOR(  7 DOWNTO 0 );
			DBO			: IN	STD_LOGIC_VECTOR(  7 DOWNTO 0 );

			JOYA		: INOUT	STD_LOGIC_VECTOR(  5 DOWNTO 0 );
			STRA		: OUT	STD_LOGIC;
			JOYB		: INOUT	STD_LOGIC_VECTOR(  5 DOWNTO 0 );
			STRB		: OUT	STD_LOGIC;

			KANA		: OUT	STD_LOGIC;
			CMTIN		: IN	STD_LOGIC;
			KEYMODE 	: IN	STD_LOGIC;

			WAVE		: OUT	STD_LOGIC_VECTOR(  7 DOWNTO 0 )
		);
	END COMPONENT;

	-- ESE-SCC+/RAM COMPATIBLE INTERNAL CARTRIDGE
	COMPONENT MEGARAM
		PORT(
			CLK21M		: IN	STD_LOGIC;
			RESET		: IN	STD_LOGIC;
			CLKENA		: IN	STD_LOGIC;
			REQ			: IN	STD_LOGIC;
			ACK			: OUT	STD_LOGIC;
			WRT			: IN	STD_LOGIC;
			ADR			: IN	STD_LOGIC_VECTOR( 15 DOWNTO 0 );
			DBI			: OUT	STD_LOGIC_VECTOR(  7 DOWNTO 0 );
			DBO			: IN	STD_LOGIC_VECTOR(  7 DOWNTO 0 );

			RAMREQ		: OUT	STD_LOGIC;
			RAMWRT		: OUT	STD_LOGIC;
			RAMADR		: OUT	STD_LOGIC_VECTOR( 19 DOWNTO 0 );
			RAMDBI		: IN	STD_LOGIC_VECTOR(  7 DOWNTO 0 );
			RAMDBO		: OUT	STD_LOGIC_VECTOR(  7 DOWNTO 0 );

			MAPSEL		: IN	STD_LOGIC_VECTOR(  1 DOWNTO 0 );	-- "0-":SCC+, "10":ASC8K, "11":ASC16K

			WAVL		: OUT	STD_LOGIC_VECTOR( 14 DOWNTO 0 );
			WAVR		: OUT	STD_LOGIC_VECTOR( 14 DOWNTO 0 )
		);
	END COMPONENT;

	-- OPLL YM-2413
	COMPONENT ESEOPLL
		PORT(
			CLK21M		: IN	STD_LOGIC;
			RESET		: IN	STD_LOGIC;
			CLKENA		: IN	STD_LOGIC;
			ENAWAIT 	: IN	STD_LOGIC;
			REQ			: IN	STD_LOGIC;
			ACK			: OUT	STD_LOGIC;
			WRT			: IN	STD_LOGIC;
			ADR			: IN	STD_LOGIC_VECTOR( 15 DOWNTO 0 );
			DBO			: IN	STD_LOGIC_VECTOR(  7 DOWNTO 0 );
			WAV			: OUT	STD_LOGIC_VECTOR(  9 DOWNTO 0 )
			);
	END COMPONENT;

	--	SYSTEM TIMER
	COMPONENT SYSTEM_TIMER
		PORT(
			CLK21M	: IN	STD_LOGIC;
			RESET	: IN	STD_LOGIC;
			REQ		: IN	STD_LOGIC;
			ACK		: OUT	STD_LOGIC;
			ADR		: IN	STD_LOGIC_VECTOR( 15 DOWNTO 0 );
			DBI		: OUT	STD_LOGIC_VECTOR(  7 DOWNTO 0 );
			DBO		: IN	STD_LOGIC_VECTOR(  7 DOWNTO 0 )
		);
	END	COMPONENT;

	--------------------------------------------------------------------------
	-- COMPONENT DEFINITION (FOR ONE CHIP MSX)
	--------------------------------------------------------------------------

	-- SD-RAM CONTROLLER
	COMPONENT SDRAMC
		PORT(
			MEMCLK			: IN	STD_LOGIC;							-- SD-RAM CLOCK
			-- CPU INTERFACE
			VIDEODLCLK		: IN	STD_LOGIC;
			VIDEODHCLK		: IN	STD_LOGIC;
			ISLTRFSH_N		: IN	STD_LOGIC;
			WRT_REQ			: IN	STD_LOGIC;
			CPUADR			: IN	STD_LOGIC_VECTOR( 22 DOWNTO 0 );
			DBO				: IN	STD_LOGIC_VECTOR(  7 DOWNTO 0 );
			RAMDBI			: OUT	STD_LOGIC_VECTOR(  7 DOWNTO 0 );
			PAUS			: IN	STD_LOGIC;
			SDPAUS			: OUT	STD_LOGIC;
			-- VDP INTERFACE
			WEVDP_N			: IN	STD_LOGIC;
			VDPADR			: IN	STD_LOGIC_VECTOR( 16 DOWNTO 0 );
			VRMDBO			: IN	STD_LOGIC_VECTOR(  7 DOWNTO 0 );
			VRMDBI			: OUT	STD_LOGIC_VECTOR( 15 DOWNTO 0 );
			-- SDRAM PORTS
			PMEMCKE			: OUT	STD_LOGIC;							-- SD-RAM CLOCK ENABLE
			PMEMCS_N		: OUT	STD_LOGIC;							-- SD-RAM CHIP SELECT
			PMEMRAS_N		: OUT	STD_LOGIC;							-- SD-RAM ROW/RAS
			PMEMCAS_N		: OUT	STD_LOGIC;							-- SD-RAM /CAS
			PMEMWE_N		: OUT	STD_LOGIC;							-- SD-RAM /WE
			PMEMUDQ			: OUT	STD_LOGIC;							-- SD-RAM UDQM
			PMEMLDQ			: OUT	STD_LOGIC;							-- SD-RAM LDQM
			PMEMBA1			: OUT	STD_LOGIC;							-- SD-RAM BANK SELECT ADDRESS 1
			PMEMBA0			: OUT	STD_LOGIC;							-- SD-RAM BANK SELECT ADDRESS 0
			PMEMADR			: OUT	STD_LOGIC_VECTOR( 11 DOWNTO 0 );	-- SD-RAM ADDRESS
			PMEMDAT			: INOUT	STD_LOGIC_VECTOR( 15 DOWNTO 0 )		-- SD-RAM DATA
		);
	END COMPONENT;

	--------------------------------------------------------------------------
	-- COMPONENT DEFINITION (ALTERA SPECIDIC COMPONENT)
	--------------------------------------------------------------------------

	-- CLOCK GENERATOR (BASE CLOCK 21.477MHZ, DRAM CLOCK 85.92MHZ)
	COMPONENT PLL4X
		PORT(
			INCLK0	: IN	STD_LOGIC := '0';	-- 21.48MHZ INPUT TO PLL    (EXTERNAL I/O PIN, FROM CRYSTAL OSCILLATOR)
			C0		: OUT	STD_LOGIC;			-- 21.48MHZ OUTPUT FROM PLL (INTERNAL LES, FOR VDP, INTERNAL-BUS, ETC.)
			C1		: OUT	STD_LOGIC;			-- 85.92MHZ OUTPUT FROM PLL (INTERNAL LES, FOR SD-RAM)
			C2		: OUT	STD_LOGIC			-- 85.92MHZ OUTPUT FROM PLL (EXTERNAL I/O PIN, FOR SD-RAM)
		);
	END COMPONENT;

	-- ASMI (EPCS4 READ/WRITE INTERFACE)
	COMPONENT CYCLONE_ASMIBLOCK
		PORT (
			DCLKIN		: IN	STD_LOGIC;		-- DCLK
			SCEIN		: IN	STD_LOGIC;		-- NCSO
			SDOIN		: IN	STD_LOGIC;		-- ASDO
			OE			: IN	STD_LOGIC;		--(1=DISABLE(HI-Z))
			DATA0OUT	: OUT	STD_LOGIC		-- DATA0
		);
	END COMPONENT;

	COMPONENT ESEPWM
		GENERIC (
			MSBI : INTEGER
		);
		PORT(
			CLK		: IN	STD_LOGIC;
			RESET	: IN	STD_LOGIC;
			DACIN	: IN	STD_LOGIC_VECTOR( MSBI DOWNTO 0 );
			DACOUT	: OUT	STD_LOGIC
		);
	END COMPONENT;

	--------------------------------------------------------------------------
	-- ƂŐ 
	--------------------------------------------------------------------------

	COMPONENT SCC_MIX_MUL
		PORT(
			A			: IN	STD_LOGIC_VECTOR( 15 DOWNTO 0 );	-- SIGNED 16BIT 
			B			: IN	STD_LOGIC_VECTOR(  2 DOWNTO 0 );	-- UNSIGNED 3BIT 
			C			: OUT	STD_LOGIC_VECTOR( 18 DOWNTO 0 )		-- SIGNED 19BIT 
		);
	END COMPONENT;

	--	LOW PASS FILTER 1
	COMPONENT LPF1
		GENERIC (
			MSBI	: INTEGER
		);
		PORT(
			CLK21M	: IN	STD_LOGIC;
			RESET	: IN	STD_LOGIC;
			CLKENA	: IN	STD_LOGIC;
			IDATA	: IN	STD_LOGIC_VECTOR( MSBI DOWNTO 0 );
			ODATA	: OUT	STD_LOGIC_VECTOR( MSBI DOWNTO 0 )
		);
	END COMPONENT;

	--	LOW PASS FILTER 2
	COMPONENT LPF2
		GENERIC (
			MSBI	: INTEGER
		);
		PORT(
			CLK21M	: IN	STD_LOGIC;
			RESET	: IN	STD_LOGIC;
			CLKENA	: IN	STD_LOGIC;
			IDATA	: IN	STD_LOGIC_VECTOR( MSBI DOWNTO 0 );
			ODATA	: OUT	STD_LOGIC_VECTOR( MSBI DOWNTO 0 )
		);
	END COMPONENT;

	-- INTERPOLATION FOR SOUND
	COMPONENT INTERPO
		GENERIC (
			MSBI	: INTEGER
		);
		PORT(
			CLK21M	: IN	STD_LOGIC;
			RESET	: IN	STD_LOGIC;
			CLKENA	: IN	STD_LOGIC;
			IDATA	: IN	STD_LOGIC_VECTOR( MSBI DOWNTO 0 );
			ODATA	: OUT	STD_LOGIC_VECTOR( MSBI DOWNTO 0 )
		);
	END COMPONENT;

	--------------------------------------------------------------------------
	-- COMPONENT DEFINITION (ALTERA DE1 SPECIDIC COMPONENT)
	--------------------------------------------------------------------------

	COMPONENT A_CODEC
		PORT(
			ICLK		: IN	STD_LOGIC;
			ISL			: IN	STD_LOGIC_VECTOR( 15 DOWNTO 0 );	-- LEFT CHANEL
			ISR			: IN	STD_LOGIC_VECTOR( 15 DOWNTO 0 );	-- RIGHT CHANEL
			OAUD_XCK	: OUT	STD_LOGIC;
			OAUD_DATA	: OUT	STD_LOGIC;
			OAUD_LRCK	: OUT	STD_LOGIC;
			OAUD_BCK	: OUT	STD_LOGIC
		);
	END COMPONENT;

	COMPONENT I2C_AV_CONFIG
		PORT(
			ICLK		: IN	STD_LOGIC;
			IRST_N		: IN	STD_LOGIC;
			OI2C_SCLK	: OUT	STD_LOGIC;
			OI2C_SDAT	: INOUT	STD_LOGIC
		);
	END COMPONENT;

	COMPONENT SEG7_LUT_4
		PORT(
			OSEG0		: OUT	STD_LOGIC_VECTOR(  6 DOWNTO 0 );
			OSEG1		: OUT	STD_LOGIC_VECTOR(  6 DOWNTO 0 );
			OSEG2		: OUT	STD_LOGIC_VECTOR(  6 DOWNTO 0 );
			OSEG3		: OUT	STD_LOGIC_VECTOR(  6 DOWNTO 0 );
			IDIG		: IN	STD_LOGIC_VECTOR( 15 DOWNTO 0 )
		);
	END COMPONENT;

	--------------------------------------------------------------------------
	--	SYSTEM TIMER
	SIGNAL	TIMREQ			: STD_LOGIC;
	SIGNAL	SYSTIM_DBI		: STD_LOGIC_VECTOR(  7 DOWNTO 0 );

	-- OPERATION MODE
	SIGNAL W_KEY_MODE		: STD_LOGIC;				-- KANA KEY BOARD LAYOUT	: 1=JIS LAYOUT
	SIGNAL FF_DIP_SW		: STD_LOGIC_VECTOR(7 DOWNTO 0);
	SIGNAL DISPMODE			: STD_LOGIC_VECTOR(1 DOWNTO 0);
	ALIAS MEGTYPE			: STD_LOGIC_VECTOR(1 DOWNTO 0) IS FF_DIP_SW(7 DOWNTO 6);
	ALIAS CLKMODE			: STD_LOGIC IS FF_DIP_SW(5);	-- '0': CPU_3.58MHZ,	'1': CPU_10.74MHZ
	ALIAS MMCMODE			: STD_LOGIC IS FF_DIP_SW(4);	-- '0': DISABLE SD/MMC, '1': ENABLE SD/MMC
	ALIAS KMAP				: STD_LOGIC IS FF_DIP_SW(3);	-- '0': JAPANESE-106,	'1': ENGLISH-101
	ALIAS SLT1MODE			: STD_LOGIC IS FF_DIP_SW(2);	-- '0': CARTRIDGE,		'1': SCC

	SIGNAL FF_DISP_SEL		: STD_LOGIC_VECTOR( 1 DOWNTO 0 );
	SIGNAL FF_RESO			: STD_LOGIC;
	SIGNAL FF_CLKMODE		: STD_LOGIC;
	SIGNAL FF_VDPMODE		: STD_LOGIC;

	-- CLOCK, RESET CONTROL SIGNALS
	SIGNAL CLK21M			: STD_LOGIC;
	SIGNAL MEMCLK			: STD_LOGIC;
	SIGNAL CPUCLK			: STD_LOGIC;
	SIGNAL CLKENA			: STD_LOGIC;
	SIGNAL CLKDIV			: STD_LOGIC_VECTOR(  1 DOWNTO 0 );
	SIGNAL FF_CLKSEL		: STD_LOGIC;
	SIGNAL RESET			: STD_LOGIC;

	-- MSX CARTRIDGE SLOT CONTROL SIGNALS
	SIGNAL BUSDIR			: STD_LOGIC;
	SIGNAL ISLTRFSH_N		: STD_LOGIC;
	SIGNAL ISLTMERQ_N		: STD_LOGIC;
	SIGNAL ISLTIORQ_N		: STD_LOGIC;
	SIGNAL XSLTRD_N			: STD_LOGIC;
	SIGNAL XSLTWR_N			: STD_LOGIC;
	SIGNAL ISLTADR			: STD_LOGIC_VECTOR(15 DOWNTO 0);
	SIGNAL ISLTDAT			: STD_LOGIC_VECTOR(7 DOWNTO 0);
	SIGNAL DLYDBI			: STD_LOGIC_VECTOR(7 DOWNTO 0);
	SIGNAL BUSREQ_N			: STD_LOGIC;
	SIGNAL CPUM1_N			: STD_LOGIC;
	SIGNAL CPURST_N			: STD_LOGIC;
	SIGNAL CPURFSH_N		: STD_LOGIC;

	-- INTERNAL BUS SIGNALS (COMMON)
	SIGNAL REQ				: STD_LOGIC;
	SIGNAL ACK				: STD_LOGIC;
	SIGNAL IACK				: STD_LOGIC;
	SIGNAL MEM				: STD_LOGIC;
	SIGNAL WRT				: STD_LOGIC;
	SIGNAL ADR				: STD_LOGIC_VECTOR( 15 DOWNTO 0 );
	SIGNAL DBI				: STD_LOGIC_VECTOR(  7 DOWNTO 0 );
	SIGNAL DBO				: STD_LOGIC_VECTOR(  7 DOWNTO 0 );

	-- PRIMARY, EXPANSION SLOT SIGNALS
	SIGNAL EXPDBI			: STD_LOGIC_VECTOR(  7 DOWNTO 0 );
	SIGNAL EXPSLOT0			: STD_LOGIC_VECTOR(  7 DOWNTO 0 );
	SIGNAL EXPSLOT3			: STD_LOGIC_VECTOR(  7 DOWNTO 0 );
	SIGNAL PRISLTNUM		: STD_LOGIC_VECTOR(  1 DOWNTO 0 );
	SIGNAL EXPSLTNUM0		: STD_LOGIC_VECTOR(  1 DOWNTO 0 );
	SIGNAL EXPSLTNUM3		: STD_LOGIC_VECTOR(  1 DOWNTO 0 );

	-- SLOT DECODE SIGNALS
	SIGNAL ISLTBOT			: STD_LOGIC;
	SIGNAL ISLTMAP			: STD_LOGIC;
	SIGNAL JSLTMEM			: STD_LOGIC;
	SIGNAL ISLTSCC1			: STD_LOGIC;
	SIGNAL JSLTSCC1			: STD_LOGIC;
	SIGNAL ISLTSCC2			: STD_LOGIC;
	SIGNAL JSLTSCC2			: STD_LOGIC;
	SIGNAL ISLTERM			: STD_LOGIC;

	-- BIOS-ROM DECODE SIGNALS
	SIGNAL ROMREQ			: STD_LOGIC;
	SIGNAL ROM_MAIN			: STD_LOGIC;
	SIGNAL ROM_OPLL			: STD_LOGIC;
	SIGNAL ROM_EXTR			: STD_LOGIC;
	SIGNAL ROM_KANJ			: STD_LOGIC;
	SIGNAL ROM_FREE1		: STD_LOGIC;
	SIGNAL ROM_FREE2		: STD_LOGIC;

	-- IPL-ROM SIGNALS
	SIGNAL ROMDBI			: STD_LOGIC_VECTOR(7 DOWNTO 0);

	-- ESE-RAM SIGNALS
	SIGNAL ERMREQ			: STD_LOGIC;
	SIGNAL ERMRAM			: STD_LOGIC;
	SIGNAL ERMWRT			: STD_LOGIC;
	SIGNAL ERMADR			: STD_LOGIC_VECTOR(19 DOWNTO 0);

	-- SD/MMC SIGNALS
	SIGNAL MMCENA			: STD_LOGIC;
	SIGNAL MMCACT			: STD_LOGIC;
	SIGNAL MMCDBI			: STD_LOGIC_VECTOR(7 DOWNTO 0);

	-- EPCS/ASMI SIGNALS
	SIGNAL EPC_CK			: STD_LOGIC;
	SIGNAL EPC_CS			: STD_LOGIC;
	SIGNAL EPC_OE			: STD_LOGIC;
	SIGNAL EPC_DI			: STD_LOGIC;
	SIGNAL EPC_DO			: STD_LOGIC;

	-- MAPPER RAM SIGNALS
	SIGNAL MAPREQ			: STD_LOGIC;
	SIGNAL MAPDBI			: STD_LOGIC_VECTOR(7 DOWNTO 0);
	SIGNAL MAPRAM			: STD_LOGIC;
	SIGNAL MAPWRT			: STD_LOGIC;
	SIGNAL MAPADR			: STD_LOGIC_VECTOR(21 DOWNTO 0);

	-- PPI(8255) SIGNALS
	SIGNAL PPIREQ			: STD_LOGIC;
	SIGNAL PPIDBI			: STD_LOGIC_VECTOR(7 DOWNTO 0);
	SIGNAL PPIPORTA			: STD_LOGIC_VECTOR(7 DOWNTO 0);
	SIGNAL PPIPORTB			: STD_LOGIC_VECTOR(7 DOWNTO 0);
	SIGNAL PPIPORTC			: STD_LOGIC_VECTOR(7 DOWNTO 0);

	-- PS/2 SIGNALS
	SIGNAL PAUS				: STD_LOGIC;
	SIGNAL RESO				: STD_LOGIC;
	SIGNAL RESO_V			: STD_LOGIC;
	SIGNAL KANA				: STD_LOGIC;
	SIGNAL CAPS				: STD_LOGIC;
	SIGNAL FKEYS			: STD_LOGIC_VECTOR(7 DOWNTO 0);

	-- CMT SIGNALS
	SIGNAL CMTIN			: STD_LOGIC;
	ALIAS CMTOUT			: STD_LOGIC IS PPIPORTC(5);

	-- 1 BIT SOUND PORT SIGNAL
	ALIAS KEYCLICK			: STD_LOGIC IS PPIPORTC(7);

	-- RTC SIGNALS
	SIGNAL RTCREQ			: STD_LOGIC;
	SIGNAL RTCDBI			: STD_LOGIC_VECTOR(7 DOWNTO 0);

	-- KANJI ROM SIGNALS
	SIGNAL KANREQ			: STD_LOGIC;
	SIGNAL KANDBI			: STD_LOGIC_VECTOR(7 DOWNTO 0);
	SIGNAL KANROM			: STD_LOGIC;
	SIGNAL KANADR			: STD_LOGIC_VECTOR(17 DOWNTO 0);

	-- VDP SIGNALS
	SIGNAL VDPREQ			: STD_LOGIC;
	SIGNAL VDPDBI			: STD_LOGIC_VECTOR(7 DOWNTO 0);
	SIGNAL VIDEOSC			: STD_LOGIC;
	SIGNAL VIDEODLCLK		: STD_LOGIC;
	SIGNAL VIDEODHCLK		: STD_LOGIC;
	SIGNAL WEVDP_N			: STD_LOGIC;
	SIGNAL VDPADR			: STD_LOGIC_VECTOR(16 DOWNTO 0);
	SIGNAL VRMDBO			: STD_LOGIC_VECTOR(7 DOWNTO 0);
	SIGNAL VRMDBI			: STD_LOGIC_VECTOR(15 DOWNTO 0);
	SIGNAL PVDPINT_N		: STD_LOGIC;

	-- VIDEO SIGNALS
	SIGNAL VIDEOR			: STD_LOGIC_VECTOR( 5 DOWNTO 0);		-- RGB_RED
	SIGNAL VIDEOG			: STD_LOGIC_VECTOR( 5 DOWNTO 0);		-- RGB_GREEN
	SIGNAL VIDEOB			: STD_LOGIC_VECTOR( 5 DOWNTO 0);		-- RGB_BLUE
	SIGNAL VIDEOHS_N		: STD_LOGIC;												-- HOLIZONTAL SYNC
	SIGNAL VIDEOVS_N		: STD_LOGIC;												-- VERTICAL SYNC
	SIGNAL VIDEOCS_N		: STD_LOGIC;												-- COMPOSITE SYNC
	SIGNAL VIDEOY			: STD_LOGIC_VECTOR( 5 DOWNTO 0);		-- SVIDEO_Y
	SIGNAL VIDEOC			: STD_LOGIC_VECTOR( 5 DOWNTO 0);		-- SVIDEO_C
	SIGNAL VIDEOV			: STD_LOGIC_VECTOR( 5 DOWNTO 0);		-- COMPOSITEVIDEO

	-- PSG SIGNALS
	SIGNAL PSGREQ			: STD_LOGIC;
	SIGNAL PSGDBI			: STD_LOGIC_VECTOR(7 DOWNTO 0);
	SIGNAL PSGAMP			: STD_LOGIC_VECTOR(7 DOWNTO 0);

	-- SCC SIGNALS
	SIGNAL SCC1REQ			: STD_LOGIC;
	SIGNAL SCC1ACK			: STD_LOGIC;
	SIGNAL SCC1DBI			: STD_LOGIC_VECTOR(  7 DOWNTO 0 );
	SIGNAL SCC1RAM			: STD_LOGIC;
	SIGNAL SCC1WRT			: STD_LOGIC;
	SIGNAL SCC1ADR			: STD_LOGIC_VECTOR( 19 DOWNTO 0 );
	SIGNAL SCC1AMPL			: STD_LOGIC_VECTOR( 14 DOWNTO 0 );

	SIGNAL SCC2REQ			: STD_LOGIC;
	SIGNAL SCC2ACK			: STD_LOGIC;
	SIGNAL SCC2DBI			: STD_LOGIC_VECTOR(  7 DOWNTO 0 );
	SIGNAL SCC2RAM			: STD_LOGIC;
	SIGNAL SCC2WRT			: STD_LOGIC;
	SIGNAL SCC2ADR			: STD_LOGIC_VECTOR( 19 DOWNTO 0 );
	SIGNAL SCC2AMPL			: STD_LOGIC_VECTOR( 14 DOWNTO 0 );

	SIGNAL SCC1TYPE			: STD_LOGIC_VECTOR(  1 DOWNTO 0 );

	-- OPLL SIGNALS
	SIGNAL OPLLREQ			: STD_LOGIC;
	SIGNAL OPLLACK			: STD_LOGIC;
	SIGNAL OPLLAMP			: STD_LOGIC_VECTOR(9 DOWNTO 0);
	SIGNAL OPLLENAWAIT		: STD_LOGIC;

	-- SOUND SIGNALS
	CONSTANT DAC_MSBI		: INTEGER := 13;
	SIGNAL DACIN			: STD_LOGIC_VECTOR(DAC_MSBI DOWNTO 0);
	SIGNAL DACOUT			: STD_LOGIC;

	SIGNAL PSGVOL			: STD_LOGIC_VECTOR(  2 DOWNTO 0 );
	SIGNAL SCCVOL			: STD_LOGIC_VECTOR(  2 DOWNTO 0 );
	SIGNAL OPLLVOL			: STD_LOGIC_VECTOR(  2 DOWNTO 0 );
	SIGNAL MSTRVOL			: STD_LOGIC_VECTOR(  2 DOWNTO 0 );

	SIGNAL PSLTSNDL			: STD_LOGIC_VECTOR(  5 DOWNTO 0 );
	SIGNAL PSLTSNDR			: STD_LOGIC_VECTOR(  5 DOWNTO 0 );
	SIGNAL PSLTSOUND		: STD_LOGIC_VECTOR(  5 DOWNTO 0 );

	-- EXERNAL MEMORY SIGNALS
	SIGNAL RAMREQ			: STD_LOGIC;
	SIGNAL RAMACK			: STD_LOGIC;
	SIGNAL RAMDBI			: STD_LOGIC_VECTOR(  7 DOWNTO 0 );
	SIGNAL CLRADR			: STD_LOGIC_VECTOR( 17 DOWNTO 0 );
	SIGNAL CPUADR			: STD_LOGIC_VECTOR( 22 DOWNTO 0 );

	-- CLOCK DIVIDER
	SIGNAL CLKDIV3			: STD_LOGIC_VECTOR(  1 DOWNTO 0 );
	SIGNAL W_10HZ			: STD_LOGIC;

	-- OPERATION MODE
	SIGNAL FF_CLK21M_CNT	: STD_LOGIC_VECTOR( 21 DOWNTO 0 );		-- FREE RUN COUNTER

	-- SOUND OUTPUT
	SIGNAL VFKEYS			: STD_LOGIC_VECTOR(  7 DOWNTO 0 );

	-- DRAM ARBITER
	SIGNAL W_WRT_REQ		: STD_LOGIC;

	-- MIXER
	SIGNAL	FF_PREPSG		: STD_LOGIC_VECTOR(  8 DOWNTO 0 );
	SIGNAL	FF_PRESCC		: STD_LOGIC_VECTOR( 15 DOWNTO 0 );
	SIGNAL	FF_PSG			: STD_LOGIC_VECTOR( DACIN'HIGH + 2 DOWNTO DACIN'LOW );
	SIGNAL	FF_SCC			: STD_LOGIC_VECTOR( DACIN'HIGH + 2 DOWNTO DACIN'LOW );
	SIGNAL	W_SCC_SFT		: STD_LOGIC_VECTOR( DACIN'HIGH + 2 DOWNTO DACIN'LOW );
	SIGNAL	W_SCC			: STD_LOGIC_VECTOR( 18 DOWNTO 0 );
	SIGNAL	W_S				: STD_LOGIC_VECTOR( 15 DOWNTO 7 );
	SIGNAL	FF_OPLL			: STD_LOGIC_VECTOR( DACIN'HIGH + 2 DOWNTO DACIN'LOW );
	SIGNAL	FF_PSG_OFFSET	: STD_LOGIC_VECTOR( DACIN'HIGH + 2 DOWNTO DACIN'LOW );
	SIGNAL	FF_SCC_OFFSET	: STD_LOGIC_VECTOR( DACIN'HIGH + 2 DOWNTO DACIN'LOW );
	SIGNAL	FF_PRE_DACIN	: STD_LOGIC_VECTOR( DACIN'HIGH + 2 DOWNTO DACIN'LOW );
	CONSTANT C_AMP_OFFSET	: STD_LOGIC_VECTOR( DACIN'HIGH + 2 DOWNTO DACIN'LOW ) := ( FF_PRE_DACIN'HIGH => '1', OTHERS => '0' );
	CONSTANT C_OPLL_ZERO	: STD_LOGIC_VECTOR( OPLLAMP'RANGE ) := ( OPLLAMP'HIGH => '1', OTHERS => '0' );

	-- SOUND OUTPUT FILTER
	SIGNAL	LPF1_WAVE		: STD_LOGIC_VECTOR( DACIN'HIGH DOWNTO 0 );
	SIGNAL	LPF5_WAVE		: STD_LOGIC_VECTOR( DACIN'HIGH DOWNTO 0 );

	SIGNAL	FF_LPF_DIV		: STD_LOGIC;
	SIGNAL	W_LPF2ENA		: STD_LOGIC;

	SIGNAL	W_PAGE_DEC		: STD_LOGIC_VECTOR(  3 DOWNTO 0 );
	SIGNAL	W_PRISLT_DEC	: STD_LOGIC_VECTOR(  3 DOWNTO 0 );
	SIGNAL	W_EXPSLT0_DEC	: STD_LOGIC_VECTOR(  3 DOWNTO 0 );
	SIGNAL	W_EXPSLT3_DEC	: STD_LOGIC_VECTOR(  3 DOWNTO 0 );

	-- ADDRESS DECODE SIGNALS
	SIGNAL W_MAPADR			: STD_LOGIC_VECTOR( 19 DOWNTO 0 );
	SIGNAL W_SCC1ADR		: STD_LOGIC_VECTOR( 19 DOWNTO 0 );
	SIGNAL W_SCC2ADR		: STD_LOGIC_VECTOR( 19 DOWNTO 0 );
	SIGNAL W_ERMADR			: STD_LOGIC_VECTOR( 19 DOWNTO 0 );
	SIGNAL W_KANADR			: STD_LOGIC_VECTOR( 19 DOWNTO 0 );
	SIGNAL W_FRM1ADR		: STD_LOGIC_VECTOR( 19 DOWNTO 0 );
	SIGNAL W_FRM2ADR		: STD_LOGIC_VECTOR( 19 DOWNTO 0 );
	SIGNAL W_MAINADR		: STD_LOGIC_VECTOR( 19 DOWNTO 0 );
	SIGNAL W_EXTRADR		: STD_LOGIC_VECTOR( 19 DOWNTO 0 );
	SIGNAL W_OPLLADR		: STD_LOGIC_VECTOR( 19 DOWNTO 0 );

	SIGNAL SDPAUS			: STD_LOGIC;

	SIGNAL PSLTCLK			: STD_LOGIC;
	SIGNAL SOUND_A			: STD_LOGIC_VECTOR( 15 DOWNTO 0 );

BEGIN

	----------------------------------------------------------------
	-- CLOCK GENERATOR (21.48MHZ > 3.58MHZ)
	-- PCPUCLK SHOULD BE INDEPENDENT FROM RESET
	----------------------------------------------------------------

	-- CLOCK ENABLER : 3.58MHZ = 21.48MHZ / 6
	PROCESS( RESET, CLK21M )
	BEGIN
		IF( RESET = '1' )THEN
			CLKENA	<= '0';
		ELSIF (CLK21M'EVENT AND CLK21M = '1') THEN
			IF( CLKDIV3 = "00" )THEN
				CLKENA <= CPUCLK;
			ELSE
				CLKENA <= '0';
			END IF;
		END IF;
	END PROCESS;

	-- CPUCLK : 3.58MHZ = 21.48MHZ / 6
	PROCESS( RESET, CLK21M )
	BEGIN
		IF( RESET = '1' )THEN
			CPUCLK	<= '1';
		ELSIF (CLK21M'EVENT AND CLK21M = '1') THEN
			IF( CLKDIV3 = "10" )THEN
				CPUCLK <= NOT CPUCLK;
			ELSE
				-- HOLD
			END IF;
		END IF;
	END PROCESS;

	-- PRESCALER : 21.48MHZ / 6
	PROCESS( RESET, CLK21M )
	BEGIN
		IF( RESET = '1' )THEN
			CLKDIV3	<= "10";
		ELSIF( CLK21M'EVENT AND CLK21M = '1' )THEN
			IF( CLKDIV3 = "00" )THEN
				CLKDIV3	<= "10";
			ELSE
				CLKDIV3	<=	CLKDIV3 - 1;
			END IF;
		END IF;
	END PROCESS;

	-- PRESCALER : 21.48MHZ / 4
	PROCESS( RESET, CLK21M )
	BEGIN
		IF( RESET = '1' )THEN
			CLKDIV	<= (OTHERS => '0');
		ELSIF( CLK21M'EVENT AND CLK21M = '1' )THEN
			CLKDIV	<=	CLKDIV - 1;
		END IF;
	END PROCESS;

	----------------------------------------------------------------
	-- CLOCK SELECTOR
	----------------------------------------------------------------
	PROCESS( RESET, CLK21M )
	BEGIN
		IF( RESET = '1' )THEN
			FF_CLKMODE <= '0';
		ELSIF( CLK21M'EVENT AND CLK21M = '1' )THEN
			IF( FKEYS(0) /= VFKEYS(0) AND FKEYS(7) = '0' )THEN			--	TOGGLE 3.58MHZ/10.74MHZ, WHEN PRESS THE [F12]
				FF_CLKMODE <= NOT FF_CLKMODE;
			END IF;
		END IF;
	END PROCESS;

	PROCESS( RESET, CLK21M )
	BEGIN
		IF( RESET = '1' )THEN
			FF_VDPMODE <= '0';
		ELSIF( CLK21M'EVENT AND CLK21M = '1' )THEN
			IF( FKEYS(0) /= VFKEYS(0) AND FKEYS(7) = '1' )THEN		--	TOGGLE VDP NORMAL/HIGHSPEED, WHEN PRESS THE [F12]+[SHIFT]
				FF_VDPMODE <= NOT FF_VDPMODE;
			END IF;
		END IF;
	END PROCESS;

	PROCESS( RESET, CLK21M )
	BEGIN
		IF( RESET = '1' )THEN
			FF_CLKSEL <= '0';						--	0 = 3.58MHZ (STANDARD SPEED), 1 = 10.74MHZ (HIGH SPEED)
		ELSIF( CLK21M'EVENT AND CLK21M = '0' )THEN
			IF( FKEYS(0) /= VFKEYS(0) )THEN			--	TOGGLE 3.58MHZ/10.74MHZ, WHEN PRESS THE [F12]
				FF_CLKSEL <= NOT FF_CLKMODE;
			ELSIF( (CPUCLK = '0') AND (CLKDIV = "00") AND (W_10HZ = '1') )THEN
				IF( CLKMODE = '0' )THEN
					FF_CLKSEL <= FF_CLKMODE;
				ELSE
					FF_CLKSEL <= NOT FF_CLKSEL;
				END IF;
			END IF;
		END IF;
	END PROCESS;

	-- FFŒ@M 1BITőI邱 (PCPUCLK ɃqQoƕsɂȂ) 
	PCPUCLK <=	CPUCLK		WHEN( FF_CLKSEL = '0' )ELSE
				CLKDIV(0);
	PSLTCLK <=	CPUCLK		WHEN( FF_CLKSEL = '0' )ELSE
				CLKDIV(0);

	RESET <= NOT PSW(0);

	----------------------------------------------------------------
	-- OPERATION MODE
	----------------------------------------------------------------

	-- 10HZ GENERATOR (FOR KEY LATCH TIMING AND RTC BASE)
	PROCESS( RESET, CLK21M )
	BEGIN
		IF( RESET = '1' )THEN
			FF_CLK21M_CNT <= (OTHERS => '0');
		ELSIF( CLK21M'EVENT AND CLK21M = '1' )THEN
			IF( W_10HZ = '1' )THEN
				FF_CLK21M_CNT <= "1000001100010101110100";	-- 21.477MHZ / 10HZ = 2147700
			ELSE
				FF_CLK21M_CNT <= FF_CLK21M_CNT - 1;
			END IF;
		END IF;
	END PROCESS;

	W_10HZ	<=	'1' WHEN( FF_CLK21M_CNT = "0000000000000000000000" )ELSE
				'0';

	-- DIP SW LATCH
	PROCESS( CLK21M )
	BEGIN
		IF( CLK21M'EVENT AND CLK21M = '1' )THEN
			IF( W_10HZ = '1' )THEN			--	CHATTERING PROTECT
				FF_DIP_SW <= NOT PDIP( 7 DOWNTO 0 );	--	CONVERT NEGATIVE LOGIC TO POSITIVE LOGIC, AND LATCH
			ELSE
				--	HOLD
			END IF;
		END IF;
	END PROCESS;
	PLEDR( 7 DOWNTO 0 ) <= FF_DIP_SW;

	W_KEY_MODE	<=	'1';					-- KANA KEY BOARD LAYOUT	: 1=JIS LAYOUT

	-- POWER LED
	PROCESS( RESET, CLK21M )
	BEGIN
		IF( RESET = '1' )THEN
			PLEDR(9)	<= '0';				-- RESET
		ELSIF( CLK21M'EVENT AND CLK21M = '1' )THEN
			PLEDR(9) <= '1';				-- POWER ON
		END IF;
	END PROCESS;

	-- INDICATOR LED
	PROCESS( CLK21M )
	BEGIN
		IF( CLK21M'EVENT AND CLK21M = '1' )THEN
			PLEDG	<=	MMCENA &								-- MMC/SD ACCESS INDICATOR
						FF_CLKSEL &								-- CPU CLOCK 10MHZ INDICATOR
						(NOT SLT1MODE) &						-- CARTRIDGE SLOT#1 ENABLE INDICATOR
						(NOT (MEGTYPE(0) OR MEGTYPE(1))) &		-- CARTRIDGE SLOT#2 ENABLE INDICATOR
						FF_VDPMODE &							-- VDP NORMAL/HIGH-SPEED INDICATOR
						"000";
		END IF;
	END PROCESS;

	----------------------------------------------------------------
	-- MSX CARTRIDGE SLOT CONTROL
	----------------------------------------------------------------
	PSLTCS1_N	<=	PSLTRD_N WHEN( PSLTADR(15 DOWNTO 14) = "01" )ELSE '1';
	PSLTCS2_N	<=	PSLTRD_N WHEN( PSLTADR(15 DOWNTO 14) = "10" )ELSE '1';
	PSLTCS12_N	<=	PSLTRD_N WHEN( PSLTADR(15 DOWNTO 14) = "01" )ELSE 
					PSLTRD_N WHEN( PSLTADR(15 DOWNTO 14) = "10" )ELSE '1';
	PSLTM1_N	<=	CPUM1_N;
	PSLTRFSH_N	<=	CPURFSH_N;

	PSLTINT_N	<=	PVDPINT_N;

	PSLTSLTSL_N <=	'1' WHEN( SCC1TYPE /= "00" )ELSE
					'0' WHEN( PSLTMERQ_N = '0' AND CPURFSH_N = '1' AND PRISLTNUM = "01" )ELSE
					'1';

	PSLTSLTS2_N <=	'1' WHEN( MEGTYPE /= "00" )ELSE
					'0' WHEN( PSLTMERQ_N	= '0' AND CPURFSH_N = '1' AND PRISLTNUM	 = "10" )ELSE
					'1';

	PSLTDAT		<=	(OTHERS => 'Z')	WHEN( PSLTRD_N = '1' )ELSE
					DBI				WHEN( PSLTIORQ_N = '0' AND BUSDIR	 = '1'  )ELSE
					DBI				WHEN( PSLTMERQ_N = '0' AND PRISLTNUM = "00" )ELSE
					DBI				WHEN( PSLTMERQ_N = '0' AND PRISLTNUM = "11" )ELSE
					DBI				WHEN( PSLTMERQ_N = '0' AND PRISLTNUM = "01" AND SCC1TYPE /= "00" )ELSE
					DBI				WHEN( PSLTMERQ_N = '0' AND PRISLTNUM = "10" AND MEGTYPE  /= "00" )ELSE
					(OTHERS => 'Z');

	PSLTBDIR_N	<= '1';
	PSLTRSV5	<= '1';
	PSLTRSV16	<= '1';
	PSLTSW1		<= '1';
	PSLTSW2		<= '1';

	----------------------------------------------------------------
	-- Z80 CPU WAIT CONTROL
	----------------------------------------------------------------
	PROCESS(PSLTCLK, RESET)

		VARIABLE ICPUM1_N	: STD_LOGIC;							-- [WARNNING!!] SLACK 1.759NS
		VARIABLE JSLTMERQ_N	: STD_LOGIC;
		VARIABLE JSLTIORQ_N	: STD_LOGIC;
		VARIABLE COUNT		: STD_LOGIC_VECTOR(3 DOWNTO 0);			-- [WARNNING!!] SLACK 0.822NS

	BEGIN

		IF (RESET = '1') THEN
			ICPUM1_N	:= '1';
			JSLTIORQ_N	:= '1';
			JSLTMERQ_N	:= '1';
			COUNT		:= (OTHERS => '0');
			PSLTWAIT_N	<= 'Z';
		ELSIF (PSLTCLK'EVENT AND PSLTCLK = '1') THEN

			IF (PSLTMERQ_N = '0' AND JSLTMERQ_N = '1') THEN
				IF( FF_CLKSEL = '1' )THEN
					COUNT := "0010";
				END IF;
			ELSIF (PSLTIORQ_N = '0' AND JSLTIORQ_N = '1') THEN
				IF( FF_CLKSEL = '1' )THEN
					COUNT := "0011";
				END IF;
			ELSIF (COUNT /= "0000") THEN
				COUNT := COUNT - 1;
			END IF;

			IF (CPUM1_N = '0' AND ICPUM1_N = '1') THEN
				PSLTWAIT_N <= '0';
			ELSIF (COUNT /= "0000") THEN
				PSLTWAIT_N <= '0';
			ELSIF (FF_CLKSEL = '1' AND OPLLREQ = '1' AND OPLLACK = '0') THEN
				PSLTWAIT_N <= '0';
			ELSIF (ERMREQ = '1' AND ADR(15 DOWNTO 13) = "010" AND MMCACT = '1') THEN
				PSLTWAIT_N <= '0';
			ELSIF (SDPAUS = '1') THEN
				PSLTWAIT_N <= '0';
			ELSE
				PSLTWAIT_N <= 'Z';
			END IF;

			ICPUM1_N := CPUM1_N;
			JSLTIORQ_N := PSLTIORQ_N;
			JSLTMERQ_N := PSLTMERQ_N;

		END IF;

	END PROCESS;

	----------------------------------------------------------------
	-- ON CHIP INTERNAL BUS CONTROL
	----------------------------------------------------------------
	PROCESS(CLK21M, RESET)

		VARIABLE EXPDEC : STD_LOGIC;

	BEGIN

		IF (RESET = '1') THEN

			ISLTRFSH_N		<= '1';
			ISLTMERQ_N		<= '1';
			ISLTIORQ_N		<= '1';
			XSLTRD_N		<= '1';
			XSLTWR_N		<= '1';
			ISLTADR			<= (OTHERS => '1');
			ISLTDAT			<= (OTHERS => '1');

			IACK			<= '0';

			DLYDBI			<= (OTHERS => '1');
			EXPDEC			:= '0';

		ELSIF( CLK21M'EVENT AND CLK21M = '1' )THEN

			-- MSX SLOT SIGNALS
			ISLTRFSH_N		<= PSLTRFSH_N;
			ISLTMERQ_N		<= PSLTMERQ_N;
			ISLTIORQ_N		<= PSLTIORQ_N;
			XSLTRD_N		<= PSLTRD_N;
			XSLTWR_N		<= PSLTWR_N;
			ISLTADR			<= PSLTADR;
			ISLTDAT			<= PSLTDAT;

			IF( ISLTMERQ_N = '1' AND ISLTIORQ_N = '1' )THEN
				IACK <= '0';
			ELSIF (ACK = '1') THEN
				IACK <= '1';
			END IF;

			IF (MEM = '1' AND EXPDEC = '1') THEN
				DLYDBI <= EXPDBI;
			ELSIF (MEM = '1' AND ISLTBOT = '1') THEN
				DLYDBI <= ROMDBI;
			ELSIF (MEM = '1' AND ISLTERM = '1' AND MMCENA = '1') THEN
				DLYDBI <= MMCDBI;
			ELSIF (MEM = '0' AND ADR(6 DOWNTO 2)	= "00010") THEN
				DLYDBI <= VDPDBI;
			ELSIF (MEM = '0' AND ADR(6 DOWNTO 2)	= "00110") THEN
				DLYDBI <= VDPDBI;
			ELSIF (MEM = '0' AND ADR(6 DOWNTO 2)	= "01000") THEN
				DLYDBI <= PSGDBI;
			ELSIF (MEM = '0' AND ADR(6 DOWNTO 2)	= "01010") THEN
				DLYDBI <= PPIDBI;
			ELSIF (MEM = '0' AND ADR(6 DOWNTO 2)	= "11111") THEN
				DLYDBI <= MAPDBI;
			ELSIF (MEM = '0' AND ADR(6 DOWNTO 1)	= "011010") THEN
				DLYDBI <= RTCDBI;
			ELSIF (MEM = '0' AND ADR(6 DOWNTO 2)	= "10110") THEN
				DLYDBI <= KANDBI;
			ELSIF (MEM = '0' AND ADR(6 DOWNTO 1)	= "110011") THEN
				DLYDBI <= SYSTIM_DBI;
			ELSE
				DLYDBI <= (OTHERS => '1');
			END IF;

			IF (ADR = X"FFFF") THEN
				EXPDEC := '1';
			ELSE
				EXPDEC := '0';
			END IF;

		END IF;

	END PROCESS;

	----------------------------------------------------------------
	PROCESS(CLK21M, RESET)

	BEGIN

		IF (RESET = '1') THEN

			JSLTSCC1	<= '0';
			JSLTSCC2	<= '0';
			JSLTMEM		<= '0';

			WRT <= '0';

		ELSIF( CLK21M'EVENT AND CLK21M = '0' )THEN

			IF( MEM = '1' AND ISLTSCC1 = '1' )THEN
				JSLTSCC1 <= '1';
			ELSE
				JSLTSCC1 <= '0';
			END IF;

			IF( MEM = '1' AND ISLTSCC2 = '1' )THEN
				JSLTSCC2 <= '1';
			ELSE
				JSLTSCC2 <= '0';
			END IF;

			IF (MEM = '1' AND ISLTERM = '1') THEN
				IF (MMCENA = '1' AND ADR(15 DOWNTO 13) = "010") THEN
					JSLTMEM <= '0';
				ELSIF (MMCMODE = '1') THEN			-- ENABLE SD/MMC DRIVE
					JSLTMEM <= '1';
				ELSE								-- DISABLE SD/MMC DRIVE
					JSLTMEM <= '0';
				END IF;
			ELSIF (MEM = '1' AND (ISLTMAP = '1' OR ROM_MAIN = '1' OR ROM_OPLL = '1' OR ROM_EXTR = '1' OR ROM_FREE1 = '1' OR ROM_FREE2 = '1')) THEN
					JSLTMEM <= '1';
			ELSE
					JSLTMEM <= '0';
			END IF;

			IF (REQ = '0') THEN
				WRT <= NOT PSLTWR_N;	 -- 1=WRITE, 0=READ
			END IF;
		END IF;
	END PROCESS;

	-- ACCESS REQUEST, CPU > COMPONENTS
	REQ <= '1' WHEN ( ( (ISLTMERQ_N = '0') OR (ISLTIORQ_N = '0') ) AND
					  ( (XSLTRD_N = '0')   OR (XSLTWR_N = '0')   ) AND IACK = '0') ELSE '0';
	MEM <= ISLTIORQ_N;		-- 1=MEMORY AREA, 0=I/O AREA
	DBO <= ISLTDAT;			-- CPU DATA (CPU > DEVICE)
	ADR <= ISLTADR;			-- CPU ADDRESS (CPU > DEVICE)

	-- ACCESS ACKNOWLEDGE, COMPONENTS > CPU
	ACK		<=	RAMACK	WHEN( RAMREQ = '1' )ELSE					-- ERMACK, MAPACK, KANACK;
				SCC1ACK	WHEN( MEM = '1' AND ISLTSCC1 = '1' )ELSE	-- SCC1ACK
				SCC2ACK	WHEN( MEM = '1' AND ISLTSCC2 = '1' )ELSE	-- SCC2ACK
				OPLLACK	WHEN( OPLLREQ = '1' )ELSE					-- OPLLACK
				REQ;												-- PSGACK, PPIACK, MAPACK, VDPACK, RTCACK
	DBI		<=	SCC1DBI	WHEN( JSLTSCC1 = '1' )ELSE
				SCC2DBI	WHEN( JSLTSCC2 = '1' )ELSE
				RAMDBI	WHEN( JSLTMEM  = '1' )ELSE
				DLYDBI;

	----------------------------------------------------------------
	-- PPI(8255) / PRIMARY-SLOT, KEYBOARD, 1 BIT SOUND PORT
	----------------------------------------------------------------
	PROCESS( RESET, CLK21M )
	BEGIN
		IF( RESET = '1' )THEN
			PPIPORTA <= "11111111";			-- PRIMARY SLOT : PAGE 0 => BOOT-ROM, PAGE 1/2 => ESE-MMC, PAGE 3 => MAPPER
			PPIPORTC <= (OTHERS => '0');
		ELSIF( CLK21M'EVENT AND CLK21M = '1' )THEN
			-- I/O PORT ACCESS ON A8-ABH ... PPI(8255) ACCESS
			IF( PPIREQ = '1' )THEN
				IF( WRT = '1' AND ADR(1 DOWNTO 0) = "00" )THEN
					PPIPORTA <= DBO;
				ELSIF( WRT = '1' AND ADR(1 DOWNTO 0) = "10" )THEN
					PPIPORTC <= DBO;
				ELSIF( WRT = '1' AND ADR(1 DOWNTO 0) = "11" AND DBO(7) = '0' )THEN
					CASE DBO(3 DOWNTO 1) IS
						WHEN "000"	=> PPIPORTC(0) <= DBO(0); -- KEY_MATRIX Y(0)
						WHEN "001"	=> PPIPORTC(1) <= DBO(0); -- KEY_MATRIX Y(1)
						WHEN "010"	=> PPIPORTC(2) <= DBO(0); -- KEY_MATRIX Y(2)
						WHEN "011"	=> PPIPORTC(3) <= DBO(0); -- KEY_MATRIX Y(3)
						WHEN "100"	=> PPIPORTC(4) <= DBO(0); -- CASSETE MOTOR ON (0=ON,1=OFF)
						WHEN "101"	=> PPIPORTC(5) <= DBO(0); -- CASSETE AUDIO OUT
						WHEN "110"	=> PPIPORTC(6) <= DBO(0); -- CAPS LAMP (0=ON,1=OFF)
						WHEN OTHERS => PPIPORTC(7) <= DBO(0); -- 1 BIT SOUND PORT
					END CASE;
				END IF;
			END IF;
		END IF;
	END PROCESS;

	CAPS <= PPIPORTC(6);

	-- I/O PORT ACCESS ON A8-ABH ... PPI(8255) REGISTER READ
	WITH ADR(1 DOWNTO 0) SELECT PPIDBI <=
		PPIPORTA			WHEN "00",
		PPIPORTB			WHEN "01",
		PPIPORTC			WHEN "10",
		(OTHERS => '1')		WHEN OTHERS;

	----------------------------------------------------------------
	-- EXPANSION SLOT
	----------------------------------------------------------------

	-- SLOT #0
	PROCESS( RESET, CLK21M )
	BEGIN
		IF( RESET = '1' )THEN
			EXPSLOT0 <= (OTHERS => '0');
		ELSIF( CLK21M'EVENT AND CLK21M = '1' )THEN
			-- MEMORY MAPPED I/O PORT ACCESS ON FFFFH ... EXPANSION SLOT REGISTER (MASTER MODE)
			IF( REQ = '1' AND ISLTMERQ_N = '0' AND WRT = '1' AND ADR = X"FFFF" )THEN
				IF( PPIPORTA(7 DOWNTO 6) = "00" )THEN
					EXPSLOT0 <= DBO;
				END IF;
			END IF;
		END IF;
	END PROCESS;

	-- SLOT #3
	PROCESS( RESET, CLK21M )
	BEGIN
		IF( RESET = '1' )THEN
			EXPSLOT3 <= "00101011";			 -- PRIMARY SLOT : PAGE 0 => IPLROM, PAGE 1/2 => MEGASD, PAGE 3 => MAPPER
		ELSIF( CLK21M'EVENT AND CLK21M = '1' )THEN
			-- MEMORY MAPPED I/O PORT ACCESS ON FFFFH ... EXPANSION SLOT REGISTER (MASTER MODE)
			IF( REQ = '1' AND ISLTMERQ_N = '0' AND WRT = '1' AND ADR = X"FFFF" )THEN
				IF( PPIPORTA(7 DOWNTO 6) = "11" )THEN
					EXPSLOT3 <= DBO;
				END IF;
			END IF;
		END IF;
	END PROCESS;

	-- PRIMARY SLOT NUMBER (MASTER MODE)
	WITH ADR(15 DOWNTO 14) SELECT PRISLTNUM	<=
		PPIPORTA(1 DOWNTO 0) WHEN "00",
		PPIPORTA(3 DOWNTO 2) WHEN "01",
		PPIPORTA(5 DOWNTO 4) WHEN "10",
		PPIPORTA(7 DOWNTO 6) WHEN OTHERS;

	-- EXPANSION SLOT NUMBER : SLOT 0 (MASTER MODE)
	WITH ADR(15 DOWNTO 14) SELECT EXPSLTNUM0 <=
		EXPSLOT0(1 DOWNTO 0) WHEN "00",
		EXPSLOT0(3 DOWNTO 2) WHEN "01",
		EXPSLOT0(5 DOWNTO 4) WHEN "10",
		EXPSLOT0(7 DOWNTO 6) WHEN OTHERS;

	-- EXPANSION SLOT NUMBER : SLOT 3 (MASTER MODE)
	WITH ADR(15 DOWNTO 14) SELECT EXPSLTNUM3 <=
		EXPSLOT3(1 DOWNTO 0) WHEN "00",
		EXPSLOT3(3 DOWNTO 2) WHEN "01",
		EXPSLOT3(5 DOWNTO 4) WHEN "10",
		EXPSLOT3(7 DOWNTO 6) WHEN OTHERS;

	-- EXPANSION SLOT REGISTER READ
	WITH PPIPORTA(7 DOWNTO 6) SELECT EXPDBI <=
		NOT EXPSLOT0		 WHEN "00",
		NOT EXPSLOT3		 WHEN "11",
		(OTHERS => '1')		 WHEN OTHERS;

	----------------------------------------------------------------
	--	SLOT/PAGE DECODE
	----------------------------------------------------------------
	WITH( ADR(15 DOWNTO 14) ) SELECT W_PAGE_DEC <=
		"0001"		WHEN "00",
		"0010"		WHEN "01",
		"0100"		WHEN "10",
		"1000"		WHEN "11",
		"XXXX"		WHEN OTHERS;

	WITH( PRISLTNUM ) SELECT W_PRISLT_DEC <=
		"0001"		WHEN "00",
		"0010"		WHEN "01",
		"0100"		WHEN "10",
		"1000"		WHEN "11",
		"XXXX"		WHEN OTHERS;

	WITH( EXPSLTNUM0 ) SELECT W_EXPSLT0_DEC <=
		"0001"		WHEN "00",
		"0010"		WHEN "01",
		"0100"		WHEN "10",
		"1000"		WHEN "11",
		"XXXX"		WHEN OTHERS;

	WITH( EXPSLTNUM3 ) SELECT W_EXPSLT3_DEC <=
		"0001"		WHEN "00",
		"0010"		WHEN "01",
		"0100"		WHEN "10",
		"1000"		WHEN "11",
		"XXXX"		WHEN OTHERS;

	----------------------------------------------------------------
	--	ADDRESS DECODE FOR CPU
	----------------------------------------------------------------
	-- SLOT0-X
	ROM_MAIN	<=	MEM	WHEN( (W_PRISLT_DEC(0) AND W_EXPSLT0_DEC(0) AND (W_PAGE_DEC(0) OR W_PAGE_DEC(1))) = '1' )ELSE
					'0';
	ROM_FREE1	<=	MEM	WHEN( (W_PRISLT_DEC(0) AND W_EXPSLT0_DEC(1)                ) = '1' AND ADR /= X"FFFF"   )ELSE
					'0';
	ROM_OPLL	<=	MEM	WHEN( (W_PRISLT_DEC(0) AND W_EXPSLT0_DEC(2) AND  W_PAGE_DEC(1)                  ) = '1' )ELSE
					'0';
	ROM_FREE2	<=	MEM	WHEN( (W_PRISLT_DEC(0) AND W_EXPSLT0_DEC(3)                ) = '1' AND ADR /= X"FFFF"   )ELSE
					'0';
	-- SLOT1
	ISLTSCC1	<=	MEM	WHEN( (W_PRISLT_DEC(1) AND (W_PAGE_DEC(1) OR W_PAGE_DEC(2))) = '1' AND SCC1TYPE /= "00" )ELSE
					'0';
	-- SLOT2
	ISLTSCC2	<=	MEM	WHEN( (W_PRISLT_DEC(2) AND (W_PAGE_DEC(1) OR W_PAGE_DEC(2))) = '1' AND MEGTYPE  /= "00" )ELSE
					'0';
	-- SLOT3-X
	ISLTMAP		<=	MEM	WHEN( (W_PRISLT_DEC(3) AND W_EXPSLT3_DEC(0)                ) = '1' AND ADR /= X"FFFF"   )ELSE
					'0';
	ROM_EXTR	<=	MEM	WHEN( (W_PRISLT_DEC(3) AND W_EXPSLT3_DEC(1) AND  W_PAGE_DEC(0)                  ) = '1' )ELSE
					'0';
	ISLTERM		<=	MEM	WHEN( (W_PRISLT_DEC(3) AND W_EXPSLT3_DEC(2) AND (W_PAGE_DEC(1) OR W_PAGE_DEC(2))) = '1' )ELSE
					'0';
	ISLTBOT		<=	MEM	WHEN( (W_PRISLT_DEC(3) AND W_EXPSLT3_DEC(3) AND (W_PAGE_DEC(0) OR W_PAGE_DEC(3))) = '1' )ELSE
					'0';
	-- I/O
	ROM_KANJ	<=	(NOT MEM)	WHEN( ADR(7 DOWNTO 2) = "110110" )ELSE
					'0';

	-- RAMX / RAMY ACCESS REQUEST
	RAMREQ	<=	SCC1RAM OR SCC2RAM OR ERMRAM OR MAPRAM OR ROMREQ OR KANROM;

	-- ACCESS REQUEST TO COMPONENT
	VDPREQ	<=	REQ WHEN( MEM = '0' AND ADR(7 DOWNTO 2) = "100110"	)ELSE '0';	-- I/O:98-9BH	/ VDP(V9958)
	PSGREQ	<=	REQ WHEN( MEM = '0' AND ADR(7 DOWNTO 2) = "101000"	)ELSE '0';	-- I/O:A0-A3H	/ PSG(AY-3-8910)
	PPIREQ	<=	REQ WHEN( MEM = '0' AND ADR(7 DOWNTO 2) = "101010"	)ELSE '0';	-- I/O:A8-ABH	/ PPI(8255)
	OPLLREQ	<=	REQ WHEN( MEM = '0' AND ADR(7 DOWNTO 2) = "011111"	)ELSE '0';	-- I/O:7C-7FH	/ OPLL(YM2413)
	KANREQ	<=	REQ WHEN( MEM = '0' AND ADR(7 DOWNTO 2) = "110110"	)ELSE '0';	-- I/O:D8-DBH	/ KANJI
	ROMREQ	<=	REQ WHEN( (ROM_MAIN OR ROM_OPLL OR ROM_EXTR OR 
									ROM_FREE1 OR ROM_FREE2) = '1'	)ELSE '0';
	MAPREQ	<=	REQ WHEN( MEM = '0' AND ADR(7 DOWNTO 2) = "111111"	)ELSE		-- I/O:FC-FFH	/ MEMORY-MAPPER
				REQ WHEN(				ISLTMAP = '1'				)ELSE '0';	-- MEM			/ MEMORY-MAPPER
	SCC1REQ	<=	REQ WHEN(				ISLTSCC1 = '1'				)ELSE '0';	-- MEM:			/ ESE-SCC
	SCC2REQ	<=	REQ WHEN(				ISLTSCC2 = '1'				)ELSE '0';	-- MEM:			/ ESE-SCC
	ERMREQ	<=	REQ WHEN(				ISLTERM = '1'				)ELSE '0';	-- MEM:			/ ESE-RAM, MEGASD
	RTCREQ	<=	REQ WHEN( MEM = '0' AND ADR(7 DOWNTO 1) = "1011010"	)ELSE '0';	-- I/O:B4-B5H	/ RTC(RP-5C01)
	TIMREQ	<=	REQ WHEN( MEM = '0' AND ADR(7 DOWNTO 1) = "1110011" )ELSE '0';	-- I/O:E6-E7H	/ SYSTEM TIMER

	BUSDIR	<=	'1' WHEN( PSLTADR(7 DOWNTO 2) = "100110"	)ELSE	-- I/O:98-9BH / VDP(V9958)
				'1' WHEN( PSLTADR(7 DOWNTO 2) = "101000"	)ELSE	-- I/O:A0-A3H / PSG(AY-3-8910)
				'1' WHEN( PSLTADR(7 DOWNTO 2) = "101010"	)ELSE	-- I/O:A8-ABH / PPI(8255)
				'1' WHEN( PSLTADR(7 DOWNTO 2) = "110110"	)ELSE	-- I/O:D8-DBH / KANJI
				'1' WHEN( PSLTADR(7 DOWNTO 2) = "111111"	)ELSE	-- I/O:FC-FFH / MEMORY-MAPPER
				'1' WHEN( PSLTADR(7 DOWNTO 1) = "1011010"	)ELSE	-- I/O:B4-B5H / RTC(RP-5C01)
				'1' WHEN( PSLTADR(7 DOWNTO 1) = "1110011"	)ELSE	-- I/O:E6-E7H / SYSTEM TIMER
				'0';

	----------------------------------------------------------------
	-- EXTERNAL MEMORY ACCESS
	----------------------------------------------------------------
	-- SLOT MAP / SD-RAM MEMORY MAP IN MASTER MODE
	--
	-- SLOT 0-0	: MAINROM				610000-617FFF(	32KB)
	-- SLOT 0-1 : DRAM1					640000-64FFFF(  64KB)
	-- SLOT 0-2	: FM-BIOS				61C000-61FFFF(	16KB)
	-- SLOT 0-3 : DRAM2					650000-65FFFF(  64KB)
	-- SLOT 1	: (EXTERNAL-SLOT)
	--				/ MEGARAM1			400000-4FFFFF(1024KB)
	-- SLOT 2	: (EXTERNAL-SLOT)
	--				/ MEGARAM2			500000-5FFFFF(1024KB)
	-- SLOT 3-0	: MAPPER				000000-1FFFFF(2048KB) OR 3FFFFF(4096KB)
	-- SLOT 3-1	: SUBROM				618000-61BFFF(	16KB)
	-- SLOT 3-2	: MEGASD				600000-60FFFF(	64KB) / 600000-6FFFFF(BIOS ETC.)
	-- SLOT 3-3	: IPL-ROM				(BLOCKRAM:512BYTES)
	-- VRAM		: VRAM					700000-71FFFF( 128KB)

	---------------------------------------------------------
	-- ISLTMAP							000000-1FFFFF(2048KB) OR 3FFFFF(4096KB)
	-- ISLTSCC1							400000-4FFFFF(1024KB)
	-- ISLTSCC2							500000-5FFFFF(1024KB)
	-- ISLTERM							600000-6FFFFF(1024KB)
	-- 									600000-60FFFF(  64KB) MEGASD(ISLTERM BANK 0-  7)
	-- 		ROM_MAIN					610000-617FFF(  32KB) MEGASD(ISLTERM BANK 8- 11)
	-- 		ROM_EXTR					618000-61BFFF(  16KB) MEGASD(ISLTERM BANK12- 13)
	-- 		ROM_OPLL					61C000-61FFFF(  16KB) MEGASD(ISLTERM BANK14- 15)
	-- 		ROM_KANJI					620000-63FFFF( 128KB) MEGASD(ISLTERM BANK16- 31)
	-- 		ROM_FRM1(SLOT0-1)			640000-64FFFF(  64KB) MEGASD(ISLTERM BANK32- 39)
	-- 		ROM_FRM2(SLOT0-3)			650000-65FFFF(  64KB) MEGASD(ISLTERM BANK40- 47)
	-- 									660000-6FFFFF( 640KB) MEGASD(ISLTERM BANK48-127)
	-- VRAM								700000-71FFFF( 128KB)
--	CPUADR(22 DOWNTO 20) <=	"00" & MAPADR(20) 					WHEN( ISLTMAP  = '1' )ELSE	--	2MB
	CPUADR(22 DOWNTO 20) <=	'0' & MAPADR(21 DOWNTO 20)	 		WHEN( ISLTMAP  = '1' )ELSE	--	4MB
							"100"								WHEN( ISLTSCC1 = '1' )ELSE	--	4XXXXX
							"101"								WHEN( ISLTSCC2 = '1' )ELSE	--	5XXXXX
							"110";															--	6XXXXX

	W_MAPADR	<=				  MAPADR(  19 DOWNTO 0 ) WHEN( ISLTMAP   = '1' )ELSE (OTHERS => '0');	--	XXXXX
	W_SCC1ADR	<=				  SCC1ADR( 19 DOWNTO 0 ) WHEN( ISLTSCC1  = '1' )ELSE (OTHERS => '0');	--	XXXXX
	W_SCC2ADR	<=				  SCC2ADR( 19 DOWNTO 0 ) WHEN( ISLTSCC2  = '1' )ELSE (OTHERS => '0');	--	XXXXX
	W_ERMADR	<=				  ERMADR(  19 DOWNTO 0 ) WHEN( ISLTERM   = '1' )ELSE (OTHERS => '0');	--	XXXXX
	W_KANADR	<=	"001"		& KANADR(  16 DOWNTO 0 ) WHEN( ROM_KANJ  = '1' )ELSE (OTHERS => '0');	--	2XXXX, 3XXXX
	W_FRM1ADR	<=	"0100"		& ADR(     15 DOWNTO 0 ) WHEN( ROM_FREE1 = '1' )ELSE (OTHERS => '0');	--	4XXXX
	W_FRM2ADR	<=	"0101"		& ADR(     15 DOWNTO 0 ) WHEN( ROM_FREE2 = '1' )ELSE (OTHERS => '0');	--	5XXXX
	W_MAINADR	<=	"00010"		& ADR(     14 DOWNTO 0 ) WHEN( ROM_MAIN  = '1' )ELSE (OTHERS => '0');	--	10XXX-17XXX
	W_EXTRADR	<=	"000110"	& ADR(     13 DOWNTO 0 ) WHEN( ROM_EXTR  = '1' )ELSE (OTHERS => '0');	--	18XXX-1BXXX
	W_OPLLADR	<=	"000111"	& ADR(     13 DOWNTO 0 ) WHEN( ROM_OPLL  = '1' )ELSE (OTHERS => '0');	--	1CXXX-1FXXX

	CPUADR(19 DOWNTO  0) <=	W_MAPADR	OR W_SCC1ADR	OR W_SCC2ADR	OR W_ERMADR		OR 
							W_KANADR	OR W_FRM1ADR	OR W_FRM2ADR	OR W_MAINADR	OR 
							W_EXTRADR	OR W_OPLLADR;


	----------------------------------------------------------------
	W_WRT_REQ	<=	(RAMREQ AND (
						(SCC1WRT AND ISLTSCC1 ) OR
						(SCC2WRT AND ISLTSCC2 ) OR
						(ERMWRT  AND ISLTERM  ) OR
						(MAPWRT  AND ISLTMAP  ))
					);

	PROCESS( RESET, CLK21M )
	BEGIN
		IF( RESET = '1' )THEN
			RAMACK <= '0';
		ELSIF( CLK21M'EVENT AND CLK21M = '1' )THEN
			IF( RAMREQ = '0' )THEN
				RAMACK <= '0';
			ELSIF( VIDEODLCLK = '0' AND VIDEODHCLK = '1' )THEN
				RAMACK <= '1';
			END IF;
		END IF;
	END PROCESS;

	U_SDRAMC: SDRAMC
	PORT MAP(
		MEMCLK			=> MEMCLK			,
		-- CPU INTERFACE
		ISLTRFSH_N		=> ISLTRFSH_N		,
		WRT_REQ			=> W_WRT_REQ		,
		CPUADR			=> CPUADR			,
		DBO				=> DBO				,
		RAMDBI			=> RAMDBI			,
		PAUS			=> PAUS				,
		SDPAUS			=> SDPAUS			,
		-- VDP INTERFACE
		VIDEODLCLK		=> VIDEODLCLK		,
		VIDEODHCLK		=> VIDEODHCLK		,
		WEVDP_N			=> WEVDP_N			,
		VDPADR			=> VDPADR			,
		VRMDBO			=> VRMDBO			,
		VRMDBI			=> VRMDBI			,
		-- SDRAM PORTS
		PMEMCKE			=> PMEMCKE			,
		PMEMCS_N		=> PMEMCS_N			,
		PMEMRAS_N		=> PMEMRAS_N		,
		PMEMCAS_N		=> PMEMCAS_N		,
		PMEMWE_N		=> PMEMWE_N			,
		PMEMUDQ			=> PMEMUDQ			,
		PMEMLDQ			=> PMEMLDQ			,
		PMEMBA1			=> PMEMBA1			,
		PMEMBA0			=> PMEMBA0			,
		PMEMADR			=> PMEMADR			,
		PMEMDAT			=> PMEMDAT			
	);

	----------------------------------------------------------------
	-- VIDEO OUTPUT
	----------------------------------------------------------------
	PROCESS (CLK21M)
	BEGIN
		IF( CLK21M'EVENT AND CLK21M = '1' )THEN
			CASE( DISPMODE + FF_DISP_SEL )IS
			WHEN "00" =>		-- TV 15KHZ
				PDAC_VR		<= VIDEOC;
				PDAC_VG		<= VIDEOY;
				PDAC_VB		<= VIDEOV;
				RESO_V		<= '0';		-- HSYNC:15KHZ
				PVIDEOHS_N	<= VIDEOHS_N;
				PVIDEOVS_N	<= VIDEOVS_N;

			WHEN "01" =>		-- RGB 15KHZ (HALF AMPLITUDE)
				PDAC_VR		<= '0' & VIDEOR( 5 DOWNTO 1 );
				PDAC_VG		<= '0' & VIDEOG( 5 DOWNTO 1 );
				PDAC_VB		<= '0' & VIDEOB( 5 DOWNTO 1 );
				RESO_V		<= '0';		-- HSYNC:15KHZ
				PVIDEOHS_N	<= VIDEOCS_N;
				PVIDEOVS_N	<= DACOUT; -- AUDIO

			WHEN "10" =>		-- VGA 31KHZ (HALF AMPLITUDE)
				PDAC_VR		<= '0' & VIDEOR( 5 DOWNTO 1 );
				PDAC_VG		<= '0' & VIDEOG( 5 DOWNTO 1 );
				PDAC_VB		<= '0' & VIDEOB( 5 DOWNTO 1 );
				RESO_V		<= '1';		-- HSYNC:31KHZ
				PVIDEOHS_N	<= VIDEOHS_N;
				PVIDEOVS_N	<= VIDEOVS_N;

			WHEN OTHERS =>	-- VGA 31KHZ (FULL AMPLITUDE)
				PDAC_VR		<= VIDEOR;
				PDAC_VG		<= VIDEOG;
				PDAC_VB		<= VIDEOB;
				RESO_V		<= '1';		-- HSYNC:31KHZ
				PVIDEOHS_N	<= VIDEOHS_N;
				PVIDEOVS_N	<= VIDEOVS_N;
			END CASE;
		END IF;
	END PROCESS;

	PROCESS( RESET, CLK21M )
	BEGIN
		IF( RESET = '1' )THEN
			FF_DISP_SEL <= ( OTHERS => '0' );
		ELSIF(CLK21M'EVENT AND CLK21M = '1') THEN
			IF( FF_RESO /= RESO )THEN
				IF( FKEYS(7) = '0' )THEN
					FF_DISP_SEL <= FF_DISP_SEL + 1;
				ELSE
					FF_DISP_SEL <= FF_DISP_SEL - 1;
				END IF;
			ELSE
				--	HOLD
			END IF;
		END IF;
	END PROCESS;

	PROCESS( RESET, CLK21M )
	BEGIN
		IF( RESET = '1' )THEN
			FF_RESO <= '0';
		ELSIF(CLK21M'EVENT AND CLK21M = '1') THEN
			FF_RESO <= RESO;
		END IF;
	END PROCESS;

	DISPMODE(1) <= FF_DIP_SW(0);
	DISPMODE(0) <= FF_DIP_SW(1);

	----------------------------------------------------------------
	-- SOUND OUTPUT
	----------------------------------------------------------------

	-- MASTER VOLUME
	PROCESS( CLK21M )
	BEGIN
		IF( CLK21M'EVENT AND CLK21M = '1' )THEN
			IF( RESET = '1' )THEN
				MSTRVOL <= "000";
			ELSIF( FKEYS(5) /= VFKEYS(5) )THEN -- MASTER VOLUME UP
				IF( MSTRVOL /= "000" )THEN
					MSTRVOL <= MSTRVOL - '1';
				END IF;
			ELSIF( FKEYS(4) /= VFKEYS(4) )THEN -- MASTER VOLUME DOWN
				IF( MSTRVOL /= "111" )THEN
					MSTRVOL <= MSTRVOL + '1';
				END IF;
			END IF;
		END IF;
	END PROCESS;

	-- PSG VOLUME
	PROCESS( CLK21M )
	BEGIN
		IF( CLK21M'EVENT AND CLK21M = '1' )THEN
			IF( RESET = '1' )THEN
				PSGVOL <= "111";	-- ORIGINAL "011";
			ELSIF( FKEYS(3) /= VFKEYS(3) )THEN
				IF( FKEYS(7) = '1' )THEN
					IF( PSGVOL /= "000" )THEN
						PSGVOL <= PSGVOL - '1';
					END IF;
				ELSE
					IF( PSGVOL /= "111" )THEN
						PSGVOL <= PSGVOL + '1';
					END IF;
				END IF;
			END IF;
		END IF;
	END PROCESS;

	-- SCC VOLUME
	PROCESS( CLK21M )
	BEGIN
		IF( CLK21M'EVENT AND CLK21M = '1' )THEN
			IF( RESET = '1' )THEN
				SCCVOL <= "111";	-- ORIGNAL "110";
			ELSIF( FKEYS(2) /= VFKEYS(2) )THEN
				IF( FKEYS(7) = '1' )THEN
					IF( SCCVOL /= "000" )THEN
						SCCVOL <= SCCVOL - '1';
					END IF;
				ELSE
					IF( SCCVOL /= "111" )THEN
						SCCVOL <= SCCVOL + '1';
					END IF;
				END IF;
			END IF;
		END IF;
	END PROCESS;

	-- OPLL VOLUME
	PROCESS( CLK21M )
	BEGIN
		IF( CLK21M'EVENT AND CLK21M = '1' )THEN
			IF( RESET = '1' )THEN
				OPLLVOL <= "111";	-- ORIGINAL "110";
			ELSIF( FKEYS(1) /= VFKEYS(1) )THEN -- OPLL
				IF( FKEYS(7) = '1' )THEN
					IF( OPLLVOL /= "000" )THEN
						OPLLVOL <= OPLLVOL - '1';
					END IF;
				ELSE
					IF( OPLLVOL /= "111" )THEN
						OPLLVOL <= OPLLVOL + '1';
					END IF;
				END IF;
			END IF;			 
		END IF;
	END PROCESS;

	PROCESS( CLK21M )
	BEGIN
		IF( CLK21M'EVENT AND CLK21M = '1' )THEN
			VFKEYS <= FKEYS;
		END IF;
	END PROCESS;

	-- MIXER (PIPE LINED)
	U_MUL: SCC_MIX_MUL
	PORT MAP (
		A	=> FF_PRESCC,	-- 16BIT ̕␔ 
		B	=> SCCVOL	,	-- 3BIT oCiij 
		C	=> W_SCC		-- 19BIT ̕␔ 
	);

	W_S <= (OTHERS => W_SCC(18));
	WITH MSTRVOL SELECT W_SCC_SFT <=
		W_S( 15 DOWNTO 14 )	& W_SCC( 18 DOWNTO  5 )	WHEN "000",
		W_S( 15 DOWNTO 13 ) & W_SCC( 18 DOWNTO  6 )	WHEN "001",
		W_S( 15 DOWNTO 12 ) & W_SCC( 18 DOWNTO  7 )	WHEN "010",
		W_S( 15 DOWNTO 11 ) & W_SCC( 18 DOWNTO  8 )	WHEN "011",
		W_S( 15 DOWNTO 10 ) & W_SCC( 18 DOWNTO  9 )	WHEN "100",
		W_S( 15 DOWNTO  9 ) & W_SCC( 18 DOWNTO 10 )	WHEN "101",
		W_S( 15 DOWNTO  8 ) & W_SCC( 18 DOWNTO 11 )	WHEN "110",
		W_S( 15 DOWNTO  7 ) & W_SCC( 18 DOWNTO 12 )	WHEN "111",
		(OTHERS => 'X') WHEN OTHERS;

	PROCESS( CLK21M )
		VARIABLE CHAMP		: STD_LOGIC_VECTOR( FF_PRE_DACIN'RANGE );
	BEGIN
		IF( CLK21M'EVENT AND CLK21M = '1' )THEN
			FF_PREPSG	<=	(('0'         & PSGAMP  ) + (KEYCLICK & "00000"));
			FF_PRESCC	<=	((SCC1AMPL(14) & SCC1AMPL) + (SCC2AMPL(14) & SCC2AMPL));

			FF_PSG		<=	"000" & SHR( (FF_PREPSG * PSGVOL) &  "0", MSTRVOL );
			FF_SCC		<=	W_SCC_SFT;

			IF( OPLLAMP < C_OPLL_ZERO )THEN
				CHAMP := "00" & SHR( ((C_OPLL_ZERO - OPLLAMP) * OPLLVOL) & "0", MSTRVOL );
				FF_OPLL <= C_AMP_OFFSET - ( CHAMP - CHAMP( CHAMP'HIGH DOWNTO 3 ) );
			ELSE
				CHAMP := "00" & SHR( ((OPLLAMP - C_OPLL_ZERO) * OPLLVOL) & "0", MSTRVOL );
				FF_OPLL <= C_AMP_OFFSET + ( CHAMP - CHAMP( CHAMP'HIGH DOWNTO 3 ) );
			END IF;
		END IF;
	END PROCESS;

	PROCESS( CLK21M )
	BEGIN
		IF( CLK21M'EVENT AND CLK21M = '1' )THEN
			FF_PRE_DACIN	<=	((FF_PSG + FF_SCC) + FF_OPLL);

			-- LIMITTER
			CASE FF_PRE_DACIN( FF_PRE_DACIN'HIGH DOWNTO FF_PRE_DACIN'HIGH - 2 ) IS
				WHEN "111" => DACIN	<= (OTHERS=>'1');
				WHEN "110" => DACIN	<= (OTHERS=>'1');
				WHEN "101" => DACIN	<= (OTHERS=>'1');
				WHEN "100" => DACIN	<= "1" & FF_PRE_DACIN( FF_PRE_DACIN'HIGH - 3 DOWNTO 0 );
				WHEN "011" => DACIN	<= "0" & FF_PRE_DACIN( FF_PRE_DACIN'HIGH - 3 DOWNTO 0 );
				WHEN "010" => DACIN	<= (OTHERS=>'0');
				WHEN "001" => DACIN	<= (OTHERS=>'0');
				WHEN "000" => DACIN	<= (OTHERS=>'0');
			END CASE;
		END IF;
	END PROCESS;

	----------------------------------------------------------------
	-- COMPONENTS
	----------------------------------------------------------------
	U_CPU : T80A
	PORT MAP(
		RESET_N		=> PSLTRST_N,
		CLK_N		=> PSLTCLK,
		WAIT_N		=> PSLTWAIT_N,
		INT_N		=> PSLTINT_N,
		NMI_N		=> '1',
		BUSRQ_N		=> BUSREQ_N,
		M1_N		=> CPUM1_N,
		MREQ_N		=> PSLTMERQ_N,
		IORQ_N		=> PSLTIORQ_N,
		RD_N		=> PSLTRD_N,
		WR_N		=> PSLTWR_N,
		RFSH_N		=> CPURFSH_N,
		HALT_N		=> OPEN,
		BUSAK_N		=> OPEN,
		A			=> PSLTADR,
		D			=> PSLTDAT
	);
	BUSREQ_N	<= '1';

	U_IPLROM: IPLROM
	PORT MAP(
		CLK			=> CLK21M,
		ADR			=> ADR,
		DBI			=> ROMDBI
	);

	U_SD_CTRL: MEGASD
	PORT MAP(
		CLK21M		=> CLK21M,
		RESET		=> RESET,
		CLKENA		=> CLKENA,
		REQ			=> ERMREQ,
		ACK			=> OPEN,
		WRT			=> WRT,
		ADR			=> ADR,
		DBI			=> OPEN,
		DBO			=> DBO,

		RAMREQ		=> ERMRAM,
		RAMWRT		=> ERMWRT,
		RAMADR		=> ERMADR,
		RAMDBI		=> RAMDBI,
		RAMDBO		=> OPEN,

		MMCDBI		=> MMCDBI,
		MMCENA		=> MMCENA,
		MMCACT		=> MMCACT,

		MMC_CK		=> PSD_CK,
		MMC_CS		=> PSD_DT3,
		MMC_DI		=> PSD_CM,
		MMC_DO		=> PSD_DT0,

		EPC_CK		=> EPC_CK,
		EPC_CS		=> EPC_CS,
		EPC_OE		=> EPC_OE,
		EPC_DI		=> EPC_DI,
		EPC_DO		=> EPC_DO
	);

	U_ASMI_CTRL: CYCLONE_ASMIBLOCK
	PORT MAP(
		DCLKIN		=> EPC_CK,
		SCEIN		=> EPC_CS,
		SDOIN		=> EPC_DI,
		OE			=> EPC_OE,
		DATA0OUT	=> EPC_DO
	);

	U_MAPPER_RAM: MAPPER
	PORT MAP(
		CLK21M		=> CLK21M,
		RESET		=> RESET,
		CLKENA		=> CLKENA,
		REQ			=> MAPREQ,
		ACK			=> OPEN,
		MEM			=> MEM,
		WRT			=> WRT,
		ADR			=> ADR,
		DBI			=> MAPDBI,
		DBO			=> DBO,

		RAMREQ		=> MAPRAM,
		RAMWRT		=> MAPWRT,
		RAMADR		=> MAPADR,
		RAMDBI		=> RAMDBI,
		RAMDBO		=> OPEN
	);

	U_PS2_KEYBOARD: ESEPS2
	PORT MAP(
		CLK21M		=> CLK21M	,
		RESET		=> RESET	,
		CLKENA		=> CLKENA	,

		KMAP		=> KMAP		,
		
		CAPS		=> CAPS		,
		KANA		=> KANA		,
		PAUS		=> PAUS		,
		RESO		=> RESO		,
		
		FKEYS		=> FKEYS	,

		PPS2CLK	 	=> PPS2CLK	,
		PPS2DAT	 	=> PPS2DAT	,
		PPIPORTC	=> PPIPORTC	,
		PKEYX		=> PPIPORTB
	);

	U_RTC: RTC
	PORT MAP(
		CLK21M		=> CLK21M	,
		RESET		=> RESET	,
		CLKENA		=> W_10HZ	,
		REQ			=> RTCREQ	,
		ACK			=> OPEN		,
		WRT			=> WRT		,
		ADR			=> ADR		,
		DBI			=> RTCDBI	,
		DBO			=> DBO		
	);

	U_KANJI_ROM : KANJI
	PORT MAP (
		CLK21M		=> CLK21M		,
		RESET		=> RESET		,
		CLKENA		=> CLKENA		,
		REQ			=> KANREQ		,
		ACK			=> OPEN			,
		WRT			=> WRT			,
		ADR			=> ADR			,
		DBI			=> KANDBI		,
		DBO			=> DBO			,

		RAMREQ		=> KANROM		,
		RAMADR		=> KANADR		,
		RAMDBI		=> RAMDBI		,
		RAMDBO		=> OPEN			
	);

	U_VDP: VDP
	PORT MAP(
		CLK21M			=> CLK21M		,
		RESET			=> RESET		,
		REQ				=> VDPREQ		,
		ACK				=> OPEN			,
		WRT				=> WRT			,
		ADR				=> ADR			,
		DBI				=> VDPDBI		,
		DBO				=> DBO			,
		INT_N			=> PVDPINT_N	,
		PRAMOE_N		=> OPEN			,
		PRAMWE_N		=> WEVDP_N		,
		PRAMADR			=> VDPADR		,
		PRAMDBI			=> VRMDBI		,
		PRAMDBO			=> VRMDBO		,
		HISPEED_MODE	=> '0'			,
		PVIDEOR			=> VIDEOR		,
		PVIDEOG			=> VIDEOG		,
		PVIDEOB			=> VIDEOB		,
		PVIDEOHS_N		=> VIDEOHS_N	,
		PVIDEOVS_N		=> VIDEOVS_N	,
		PVIDEOCS_N		=> VIDEOCS_N	,
		PVIDEODHCLK		=> VIDEODHCLK	,
		PVIDEODLCLK		=> VIDEODLCLK	,
		DISPRESO		=> RESO_V		
	);

	U_VIDEO_ENCODER: VENCODE
	PORT MAP (
		CLK21M		=> CLK21M			,
		RESET		=> RESET			,
		VIDEOR		=> VIDEOR			,
		VIDEOG		=> VIDEOG			,
		VIDEOB		=> VIDEOB			,
		VIDEOHS_N 	=> VIDEOHS_N		,
		VIDEOVS_N 	=> VIDEOVS_N		,
		VIDEOY		=> VIDEOY			,
		VIDEOC		=> VIDEOC			,
		VIDEOV		=> VIDEOV			
	);

	----------------------------------------------------------------
	-- CASSETTE MAGNETIC TAPE (CMT) INTERFACE
	----------------------------------------------------------------
--    PCMT_OUT <= CMTOUT;
--    CMTIN <= PCMT_IN;
	CMTIN <= '0';		-- CMT DATA INPUT : ALWAYS '0' ON MSX TURBOR

	U_PSG : PSG
	PORT MAP(
		CLK21M		=> CLK21M		,
		RESET		=> RESET		,
		CLKENA		=> CLKENA		,
		REQ			=> PSGREQ		,
		ACK			=> OPEN			,
		WRT			=> WRT			,
		ADR			=> ADR			,
		DBI			=> PSGDBI		,
		DBO			=> DBO			,

		JOYA		=> PJOYA		,
		STRA		=> PSTRA		,
		JOYB		=> PJOYB		,
		STRB		=> PSTRB		,

		KANA		=> KANA			,
		CMTIN		=> CMTIN		,
		KEYMODE 	=> W_KEY_MODE	,

		WAVE		=> PSGAMP
	);

	U_MEGARAM1: MEGARAM
	PORT MAP (
		CLK21M		=> CLK21M		,
		RESET		=> RESET		,
		CLKENA		=> CLKENA		,
		REQ			=> SCC1REQ		,
		ACK			=> SCC1ACK		,
		WRT			=> WRT			,
		ADR			=> ADR			,
		DBI			=> SCC1DBI		,
		DBO			=> DBO			,
		RAMREQ		=> SCC1RAM		,
		RAMWRT		=> SCC1WRT		,
		RAMADR		=> SCC1ADR		,
		RAMDBI		=> RAMDBI		,
		RAMDBO		=> OPEN			,
		MAPSEL		=> SCC1TYPE		,
		WAVL		=> SCC1AMPL		,
		WAVR		=> OPEN			
	);

	SCC1TYPE <=	"00"	WHEN( SLT1MODE = '0' )ELSE
				"10";

	U_MEGARAM2: MEGARAM
	PORT MAP (
		CLK21M		=> CLK21M		,
		RESET		=> RESET		,
		CLKENA		=> CLKENA		,
		REQ			=> SCC2REQ		,
		ACK			=> SCC2ACK		,
		WRT			=> WRT			,
		ADR			=> ADR			,
		DBI			=> SCC2DBI		,
		DBO			=> DBO			,
		RAMREQ		=> SCC2RAM		,
		RAMWRT		=> SCC2WRT		,
		RAMADR		=> SCC2ADR		,
		RAMDBI		=> RAMDBI		,
		RAMDBO		=> OPEN			,
		MAPSEL		=> MEGTYPE		,
		WAVL		=> SCC2AMPL		,
		WAVR		=> OPEN			
	);

	U_OPLL: ESEOPLL
	PORT MAP (
		CLK21M		=> CLK21M		,
		RESET		=> RESET		,
		CLKENA		=> CLKENA		,
		ENAWAIT 	=> OPLLENAWAIT	,
		REQ			=> OPLLREQ		,
		ACK			=> OPLLACK		,
		WRT			=> WRT			,
		ADR			=> ADR			,
		DBO			=> DBO			,
		WAV			=> OPLLAMP		
	);

--	OPLLENAWAIT <=	'1' WHEN( FF_CLKSEL = '1' )ELSE
--					'0';
	OPLLENAWAIT <=	'0';

	--	SOUND OUTPUT LOWPASS FILTER
	PROCESS( RESET, CLK21M )
	BEGIN
		IF( RESET = '1' )THEN
			FF_LPF_DIV <= '0';
		ELSIF( CLK21M'EVENT AND CLK21M = '1' )THEN
			IF( CLKENA = '1' )THEN
				FF_LPF_DIV <= NOT FF_LPF_DIV;
			END IF;
		END IF;
	END PROCESS;

	U_INTERPO: INTERPO
	GENERIC MAP (
		MSBI	=> DACIN'HIGH
	)
	PORT MAP (
		CLK21M	=> CLK21M		,
		RESET	=> RESET		,
		CLKENA	=> CLKENA		,
		IDATA	=> DACIN		,
		ODATA	=> LPF1_WAVE	
	);

	--	LOW PASS FILTER 
	W_LPF2ENA <= FF_LPF_DIV AND CLKENA;

	U_LPF2: LPF2
	GENERIC MAP (
		MSBI	=> DACIN'HIGH
	)
	PORT MAP (
		CLK21M	=> CLK21M		,
		RESET	=> RESET		,
		CLKENA	=> W_LPF2ENA	,
		IDATA	=> LPF1_WAVE	,
		ODATA	=> LPF5_WAVE	
	);

	U_CONVERT_TO_1BIT: ESEPWM
	GENERIC MAP(
		MSBI 	=> DAC_MSBI
	)
	PORT MAP(
		CLK		=> CLK21M,
		RESET	=> RESET,
		DACIN	=> LPF5_WAVE,
		DACOUT	=> DACOUT
	);

	U_SYSTEM_TIMER: SYSTEM_TIMER
	PORT MAP (
		CLK21M	=> CLK21M		,
		RESET	=> RESET		,
		REQ		=> TIMREQ		,
		ACK		=> OPEN			,
		ADR		=> ADR			,
		DBI		=> SYSTIM_DBI	,
		DBO		=> DBO			
	);

	----------------------------------------------------------------
	-- ALTERA DE1 BOARD ONLY
	----------------------------------------------------------------
	U_CLOCK : PLL4X
	PORT MAP(
		INCLK0	=> PCLK27M,        -- 27 MHZ EXTERNAL
		C0		=> CLK21M,         -- 21.00MHZ INTERNAL
		C1		=> MEMCLK,         -- 84.00MHZ = 21.00MHZ X 4
		C2		=> PMEMCLK         -- 84.00MHZ EXTERNAL
	);

	U_7SEG_LED: SEG7_LUT_4
	PORT MAP (
		OSEG0	=> HEX0,
		OSEG1	=> HEX1,
		OSEG2	=> HEX2,
		OSEG3	=> HEX3,
		IDIG	=> PPIPORTA & FKEYS
	);

	U_AUDIO_CODEC: A_CODEC
	PORT MAP (
		ICLK		=> CLOCK_24,
		ISL			=> SOUND_A,
		ISR			=> SOUND_A,
		OAUD_XCK	=> AUD_XCK,
		OAUD_DATA	=> AUD_DACDAT,
		OAUD_LRCK	=> AUD_DACLRCK,
		OAUD_BCK	=> AUD_BCLK
	);

	U_AV_CONFIG: I2C_AV_CONFIG
	PORT MAP (
		ICLK		=> CLOCK_24,
		IRST_N		=> NOT RESET,
		OI2C_SCLK	=> I2C_SCLK,
		OI2C_SDAT	=> I2C_SDAT
	);

	-- SOUND OUTPUT [BAD CONNECTION!!]
    PDAC_S		<= DACOUT;
    SOUND_A		<= "00" & LPF5_WAVE;

    PSD_DT0 <= 'Z';

end rtl;
