-- --------------------------------------------------------- --
--	RTC TEST BENCH											 --
-- ========================================================= --
--	COPYRIGHT (C)2008 T.HARA								 --
-- --------------------------------------------------------- --

LIBRARY	IEEE;
	USE	IEEE.STD_LOGIC_1164.ALL;
	USE	IEEE.STD_LOGIC_UNSIGNED.ALL;
	USE	IEEE.STD_LOGIC_ARITH.ALL;

ENTITY TB IS
END TB;

ARCHITECTURE BEHAVIOR OF TB IS

	-- TEST TARGET
	COMPONENT RTC
		PORT(
			CLK21M		: IN	STD_LOGIC;
			RESET		: IN	STD_LOGIC;
			CLKENA		: IN	STD_LOGIC;			-- 10HZ
			REQ			: IN	STD_LOGIC;
			ACK			: OUT	STD_LOGIC;
			WRT			: IN	STD_LOGIC;
			ADR			: IN	STD_LOGIC_VECTOR( 15 DOWNTO 0 );
			DBI			: OUT	STD_LOGIC_VECTOR(  7 DOWNTO 0 );
			DBO			: IN	STD_LOGIC_VECTOR(  7 DOWNTO 0 )
	 );
	END COMPONENT;

	CONSTANT CYCLE : TIME := 10 NS;

	SIGNAL	CLK21M			: STD_LOGIC;
	SIGNAL	RESET			: STD_LOGIC;
	SIGNAL	CLKENA			: STD_LOGIC;			-- 10HZ
	SIGNAL	REQ				: STD_LOGIC;
	SIGNAL	ACK				: STD_LOGIC;
	SIGNAL	WRT				: STD_LOGIC;
	SIGNAL	ADR				: STD_LOGIC_VECTOR( 15 DOWNTO 0 );
	SIGNAL	DBI				: STD_LOGIC_VECTOR(  7 DOWNTO 0 );
	SIGNAL	DBO				: STD_LOGIC_VECTOR(  7 DOWNTO 0 );

	SIGNAL	TB_CLKCNT		: INTEGER := 0;
	SIGNAL	TB_CLKCNT_CLR	: STD_LOGIC := '1';
	SIGNAL	TB_END			: STD_LOGIC := '0';
BEGIN

	--	INSTANCE
	U_TARGET: RTC
	PORT MAP(
		CLK21M		=> CLK21M		,
		RESET		=> RESET		,
		CLKENA		=> CLKENA		,
		REQ			=> REQ			,
		ACK			=> ACK			,
		WRT			=> WRT			,
		ADR			=> ADR			,
		DBI			=> DBI			,
		DBO			=> DBO			
	 );

	-- ----------------------------------------------------- --
	--	CLOCK GENERATOR										 --
	-- ----------------------------------------------------- --
	PROCESS
	BEGIN
		IF( TB_END = '1' )THEN
			WAIT;
		END IF;
		CLK21M <= '0';
		WAIT FOR 5 NS;
		CLK21M <= '1';
		WAIT FOR 5 NS;
	END PROCESS;

	PROCESS( CLK21M )
	BEGIN
		IF( CLK21M'EVENT AND CLK21M = '1' )THEN
			IF( TB_CLKCNT_CLR = '1' )THEN
				TB_CLKCNT <= 0;
			ELSIF( CLKENA = '1' )THEN
				TB_CLKCNT <= TB_CLKCNT + 1;
			END IF;
		END IF;
	END PROCESS;

	-- ----------------------------------------------------- --
	--	TEST BENCH											 --
	-- ----------------------------------------------------- --
	PROCESS
	BEGIN
		-- INIT
		RESET	<= '1';
		CLKENA	<= '0';
		REQ		<= '0';
		WRT		<= '0';
		ADR		<= (OTHERS => '0');
		DBO		<= (OTHERS => '0');
		WAIT UNTIL( CLK21M'EVENT AND CLK21M = '1' );
		RESET	<= '0';
		WAIT UNTIL( CLK21M'EVENT AND CLK21M = '1' );

		-- REGISTER WRITE
		REQ		<= '1';
		WRT		<= '1';
		ADR(0)	<= '0';
		DBO		<= "00001101";		--	REGPTR = 13;
		WAIT UNTIL( CLK21M'EVENT AND CLK21M = '1' );
		REQ		<= '0';
		WRT		<= '0';
		WAIT UNTIL( CLK21M'EVENT AND CLK21M = '1' );

		REQ		<= '1';
		WRT		<= '1';
		ADR(0)	<= '1';
		DBO		<= "00000000";		--	REG_MODE = "0000";
		WAIT UNTIL( CLK21M'EVENT AND CLK21M = '1' );
		REQ		<= '0';
		WRT		<= '0';
		WAIT UNTIL( CLK21M'EVENT AND CLK21M = '1' );

		-- REGISTER WRITE
		REQ		<= '1';
		WRT		<= '1';
		ADR(0)	<= '0';
		DBO		<= "00000000";		--	REGPTR = 0;
		WAIT UNTIL( CLK21M'EVENT AND CLK21M = '1' );
		REQ		<= '0';
		WRT		<= '0';
		WAIT UNTIL( CLK21M'EVENT AND CLK21M = '1' );

		REQ		<= '1';
		WRT		<= '1';
		ADR(0)	<= '1';
		DBO		<= "00000000";		--	REG_SEC_L = 0;
		WAIT UNTIL( CLK21M'EVENT AND CLK21M = '1' );
		REQ		<= '0';
		WRT		<= '0';
		WAIT UNTIL( CLK21M'EVENT AND CLK21M = '1' );

		-- REGISTER WRITE
		REQ		<= '1';
		WRT		<= '1';
		ADR(0)	<= '0';
		DBO		<= "00000001";		--	REGPTR = 1;
		WAIT UNTIL( CLK21M'EVENT AND CLK21M = '1' );
		REQ		<= '0';
		WRT		<= '0';
		WAIT UNTIL( CLK21M'EVENT AND CLK21M = '1' );

		REQ		<= '1';
		WRT		<= '1';
		ADR(0)	<= '1';
		DBO		<= "00000000";		--	REG_SEC_H = 0;
		WAIT UNTIL( CLK21M'EVENT AND CLK21M = '1' );
		REQ		<= '0';
		WRT		<= '0';
		WAIT UNTIL( CLK21M'EVENT AND CLK21M = '1' );

		-- REGISTER WRITE
		REQ		<= '1';
		WRT		<= '1';
		ADR(0)	<= '0';
		DBO		<= "00000010";		--	REGPTR = 2;
		WAIT UNTIL( CLK21M'EVENT AND CLK21M = '1' );
		REQ		<= '0';
		WRT		<= '0';
		WAIT UNTIL( CLK21M'EVENT AND CLK21M = '1' );

		REQ		<= '1';
		WRT		<= '1';
		ADR(0)	<= '1';
		DBO		<= "00000000";		--	REG_MIN_L = 0;
		WAIT UNTIL( CLK21M'EVENT AND CLK21M = '1' );
		REQ		<= '0';
		WRT		<= '0';
		WAIT UNTIL( CLK21M'EVENT AND CLK21M = '1' );

		-- REGISTER WRITE
		REQ		<= '1';
		WRT		<= '1';
		ADR(0)	<= '0';
		DBO		<= "00000011";		--	REGPTR = 3;
		WAIT UNTIL( CLK21M'EVENT AND CLK21M = '1' );
		REQ		<= '0';
		WRT		<= '0';
		WAIT UNTIL( CLK21M'EVENT AND CLK21M = '1' );

		REQ		<= '1';
		WRT		<= '1';
		ADR(0)	<= '1';
		DBO		<= "00000000";		--	REG_MIN_H = 0;
		WAIT UNTIL( CLK21M'EVENT AND CLK21M = '1' );
		REQ		<= '0';
		WRT		<= '0';
		WAIT UNTIL( CLK21M'EVENT AND CLK21M = '1' );

		-- REGISTER WRITE
		REQ		<= '1';
		WRT		<= '1';
		ADR(0)	<= '0';
		DBO		<= "00000100";		--	REGPTR = 4;
		WAIT UNTIL( CLK21M'EVENT AND CLK21M = '1' );
		REQ		<= '0';
		WRT		<= '0';
		WAIT UNTIL( CLK21M'EVENT AND CLK21M = '1' );

		REQ		<= '1';
		WRT		<= '1';
		ADR(0)	<= '1';
		DBO		<= "00000000";		--	REG_HOU_L = 0;
		WAIT UNTIL( CLK21M'EVENT AND CLK21M = '1' );
		REQ		<= '0';
		WRT		<= '0';
		WAIT UNTIL( CLK21M'EVENT AND CLK21M = '1' );

		-- REGISTER WRITE
		REQ		<= '1';
		WRT		<= '1';
		ADR(0)	<= '0';
		DBO		<= "00000101";		--	REGPTR = 5;
		WAIT UNTIL( CLK21M'EVENT AND CLK21M = '1' );
		REQ		<= '0';
		WRT		<= '0';
		WAIT UNTIL( CLK21M'EVENT AND CLK21M = '1' );

		REQ		<= '1';
		WRT		<= '1';
		ADR(0)	<= '1';
		DBO		<= "00000000";		--	REG_HOU_H = 0;
		WAIT UNTIL( CLK21M'EVENT AND CLK21M = '1' );
		REQ		<= '0';
		WRT		<= '0';
		WAIT UNTIL( CLK21M'EVENT AND CLK21M = '1' );

		-- REGISTER WRITE
		REQ		<= '1';
		WRT		<= '1';
		ADR(0)	<= '0';
		DBO		<= "00001101";		--	REGPTR = 13;
		WAIT UNTIL( CLK21M'EVENT AND CLK21M = '1' );
		REQ		<= '0';
		WRT		<= '0';
		WAIT UNTIL( CLK21M'EVENT AND CLK21M = '1' );

		REQ		<= '1';
		WRT		<= '1';
		ADR(0)	<= '1';
		DBO		<= "00001000";		--	REG_MODE = "1000";
		WAIT UNTIL( CLK21M'EVENT AND CLK21M = '1' );
		REQ		<= '0';
		WRT		<= '0';
		WAIT UNTIL( CLK21M'EVENT AND CLK21M = '1' );

		--
		FOR I IN 0 TO 5000 LOOP
			CLKENA	<= '1';
			WAIT UNTIL( CLK21M'EVENT AND CLK21M = '1' );
			CLKENA	<= '0';
			WAIT UNTIL( CLK21M'EVENT AND CLK21M = '1' );
		END LOOP;

		TB_END <= '1';
		WAIT;
	END PROCESS;

END BEHAVIOR;
