-- 
-- esedsig.vhd
--   Filtered Delta Sigma Modulation D/A Conveter
--   Revision 1.00
-- 
-- Copyright (c) 2007 Takayuki Hara
-- All rights reserved.
-- 
-- Redistribution and use of this source code or any derivative works, are 
-- permitted provided that the following conditions are met:
--
-- 1. Redistributions of source code must retain the above copyright notice, 
--    this list of conditions and the following disclaimer.
-- 2. Redistributions in binary form must reproduce the above copyright 
--    notice, this list of conditions and the following disclaimer in the 
--    documentation and/or other materials provided with the distribution.
-- 3. Redistributions may not be sold, nor may they be used in a commercial 
--    product or activity without specific prior written permission.
--
-- THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
-- "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED 
-- TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR 
-- PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR 
-- CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
-- EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
-- PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
-- OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
-- WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR 
-- OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF 
-- ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
-- 

library	ieee;
	use	ieee.std_logic_1164.all;
	use	ieee.std_logic_unsigned.all;

entity esedsig is
	generic (
		in_width	: integer;
		out_width	: integer
	);
	port (
		clk		: in	std_logic;
		reset	: in	std_logic;
		dacin	: in	std_logic_vector( in_width-1  downto 0 );
		dacout	: out	std_logic_vector( out_width-1 downto 0 )
	);
end esedsig;

architecture rtl of esedsig is
	component esefir5
		generic (
			msbi : integer
		);
		port ( 
			clk		: in	std_logic;
			reset	: in	std_logic;
			wavin	: in	std_logic_vector ( msbi downto 0 );
			wavout	: out	std_logic_vector ( msbi downto 0 )  
		);
	end component;

	signal fir_out		: std_logic_vector( in_width-1 downto 0 );
	signal ff_sigma		: std_logic_vector( in_width-out_width downto 0 );
	signal w_delta		: std_logic_vector( in_width downto 0 );
	signal w_sigma		: std_logic_vector( in_width downto 0 );
	signal w_shift		: std_logic_vector( out_width-1 downto 0 );
	constant c_zero		: std_logic_vector( in_width downto in_width-out_width+1 ) := (others => '0');
begin
	u_filter: esefir5
	generic map (
		msbi	=> in_width-1
	)
	port map (
		clk		=> clk,
		reset	=> reset,
		wavin	=> dacin,
		wavout	=> fir_out
	);

	w_sigma		<= ('0' & fir_out) + (c_zero & ff_sigma);
	process( reset, clk )
	begin
		if( reset = '1' )then
			ff_sigma	<= (others => '0');
			dacout		<= (others => '0');
		elsif( clk'event and clk = '1' )then
			ff_sigma	<= w_sigma( in_width-out_width downto 0 );
			dacout		<= w_sigma( in_width downto in_width-out_width+1 );
		end if;
	end process;

end rtl;
