-- --------------------------------------------------------- --
--	interpo test bench										 --
-- ========================================================= --
--	Copyright (c)2007 t.hara								 --
-- --------------------------------------------------------- --

library	ieee;
use		ieee.std_logic_1164.all;
use		ieee.std_logic_unsigned.all;
use		ieee.std_logic_arith.all;

entity tb is
end tb;

architecture behavior of tb is

	-- test target
	component interpo
		generic (
			msbi	: integer
		);
		port(
			clk21m	: in	std_logic;
			reset	: in	std_logic;
			clkena	: in	std_logic;
			idata	: in	std_logic_vector( msbi downto 0 );
			odata	: out	std_logic_vector( msbi downto 0 )
		);
	end component;

	constant CYCLE : time := 10 ns;

	signal clk21m : std_logic;
	signal clkena : std_logic;
	signal reset  : std_logic;
	signal wavin  : std_logic_vector( 11 downto 0 );
	signal wavout : std_logic_vector( 11 downto 0 );
	signal ff_cnt : std_logic_vector(  2 downto 0 );

	signal	tb_end	: std_logic := '0';
begin

	--	instance
	u_target: interpo
	generic map (
		msbi	=> 11
	)
	port map (
		clk21m	=> clk21m	,
		reset	=> reset	,
		clkena	=> clkena	,
		idata	=> wavin	,
		odata	=> wavout	
	);

	-- ----------------------------------------------------- --
	--	clock generator										 --
	-- ----------------------------------------------------- --
	process
	begin
		if( tb_end = '1' )then
			wait;
		end if;
		clk21m <= '0';
		wait for 5 ns;
		clk21m <= '1';
		wait for 5 ns;
	end process;

	-- ----------------------------------------------------- --
	process( reset, clk21m )
	begin
		if( reset = '1' )then
			ff_cnt <= "000";
		elsif( clk21m'event and clk21m = '1' )then
			if( ff_cnt = "101" )then
				ff_cnt <= "000";
			else
				ff_cnt <= ff_cnt + 1;
			end if;
		end if;
	end process;

	clkena <= '1' when( ff_cnt = "101" )else
			  '0';

	-- ----------------------------------------------------- --
	--	test bench											 --
	-- ----------------------------------------------------- --
	process
	begin
		-- init
		reset <= '1';
	    wavin  <= (others => '0');

		-- reset
		wait until( clk21m'event and clk21m = '1');
		reset <= '0';
		wait until( clk21m'event and clk21m = '1');

		for i in 0 to 600 loop
			if( clkena = '1' )then
				wavin <= wavin + 100;
			end if;
			wait until( clk21m'event and clk21m = '1');
		end loop;

		for i in 0 to 600 loop
			if( clkena = '1' )then
				wavin <= wavin - 10;
			end if;
			wait until( clk21m'event and clk21m = '1');
		end loop;

		tb_end <= '1';
		wait;
	end process;

end behavior;
