-- --------------------------------------------------------- --
--	SineTable test bench									 --
-- ========================================================= --
--	Copyright (c)2007 t.hara								 --
-- --------------------------------------------------------- --

library	ieee;
use		ieee.std_logic_1164.all;
use		ieee.std_logic_unsigned.all;
use		ieee.std_logic_arith.all;

entity tb is
end tb;

architecture behavior of tb is

	-- test target
	component SineTable 
	  port (
		clk		: in	std_logic;
		clkena	: in	std_logic;
		wf		: in	std_logic;
		addr	: in	std_logic_vector( 17 downto 0 );	--	 9bit,  9bit 
		data	: out	std_logic_vector( 13 downto 0 )		--	 8bit,  6bit 
	  );
	end component;

	constant CYCLE : time := 10 ns;

	signal	clk		: std_logic;
	signal	clkena	: std_logic;
	signal	wf		: std_logic;
	signal	addr	: std_logic_vector( 17 downto 0 );
	signal	data	: std_logic_vector( 13 downto 0 );

	signal	tb_end	: std_logic := '0';
begin

	--	instance
	u_target: SineTable 
	port map (
		clk     => clk     ,
		clkena	=> clkena  ,
		wf      => wf      ,
		addr    => addr    ,
		data    => data    
	);

	-- ----------------------------------------------------- --
	--	clock generator										 --
	-- ----------------------------------------------------- --
	process
	begin
		if( tb_end = '1' )then
			wait;
		end if;
		clk <= '0';
		wait for 5 ns;
		clk <= '1';
		wait for 5 ns;
	end process;

	clkena	<= '1';

	-- ----------------------------------------------------- --
	--	test bench											 --
	-- ----------------------------------------------------- --
	process
	begin
		-- init
		wf		<= '0';
		addr	<=	(others => '0');
		wait until( clk'event and clk = '1' );

		for i in 0 to 262143 loop
			addr	<=	conv_std_logic_vector( i, 18 );
			wait until( clk'event and clk = '1' );
		end loop;

		wf		<= '1';
		for i in 0 to 262143 loop
			addr	<=	conv_std_logic_vector( i, 18 );
			wait until( clk'event and clk = '1' );
		end loop;

		tb_end <= '1';
		wait;
	end process;

end behavior;
