-- --------------------------------------------------------- --
--	TEST BENCH												 --
-- ========================================================= --
--	COPYRIGHT (C)2008 T.HARA								 --
-- --------------------------------------------------------- --

LIBRARY	IEEE;
	USE	IEEE.STD_LOGIC_1164.ALL;
	USE	IEEE.STD_LOGIC_UNSIGNED.ALL;
	USE	IEEE.STD_LOGIC_ARITH.ALL;

ENTITY TB IS
END TB;

ARCHITECTURE BEHAVIOR OF TB IS

	-- TEST TARGET
	COMPONENT VDP_HVCOUNTER
		PORT(
			RESET				: IN	STD_LOGIC;
			CLK21M				: IN	STD_LOGIC;

			H_CNT				: OUT	STD_LOGIC_VECTOR( 10 DOWNTO 0 );
			V_CNT_IN_FIELD		: OUT	STD_LOGIC_VECTOR(  9 DOWNTO 0 );
			V_CNT_IN_FRAME		: OUT	STD_LOGIC_VECTOR( 10 DOWNTO 0 );
			FIELD				: OUT	STD_LOGIC;
			H_BLANK				: OUT	STD_LOGIC;
			V_BLANK				: OUT	STD_LOGIC;

			PAL_MODE			: IN	STD_LOGIC;
			INTERLACE_MODE		: IN	STD_LOGIC;
			Y212_MODE			: IN	STD_LOGIC
		);
	END COMPONENT;

	CONSTANT CYCLE : TIME := 10 NS;

	SIGNAL RESET				: STD_LOGIC;
	SIGNAL CLK21M				: STD_LOGIC;
	SIGNAL H_CNT				: STD_LOGIC_VECTOR( 10 DOWNTO 0 );
	SIGNAL V_CNT_IN_FIELD		: STD_LOGIC_VECTOR(  9 DOWNTO 0 );
	SIGNAL V_CNT_IN_FRAME		: STD_LOGIC_VECTOR( 10 DOWNTO 0 );
	SIGNAL FIELD				: STD_LOGIC;
	SIGNAL H_BLANK				: STD_LOGIC;
	SIGNAL V_BLANK				: STD_LOGIC;
	SIGNAL PAL_MODE				: STD_LOGIC;
	SIGNAL INTERLACE_MODE		: STD_LOGIC;
	SIGNAL Y212_MODE			: STD_LOGIC;

	SIGNAL TB_END				: STD_LOGIC := '0';
BEGIN

	--	INSTANCE
	U_TARGET: VDP_HVCOUNTER
	PORT MAP (
		RESET				=> RESET				,
		CLK21M				=> CLK21M				,

		H_CNT				=> H_CNT				,
		V_CNT_IN_FIELD		=> V_CNT_IN_FIELD		,
		V_CNT_IN_FRAME		=> V_CNT_IN_FRAME		,
		FIELD				=> FIELD				,
		H_BLANK				=> H_BLANK				,
		V_BLANK				=> V_BLANK				,

		PAL_MODE			=> PAL_MODE				,
		INTERLACE_MODE		=> INTERLACE_MODE		,
		Y212_MODE			=> Y212_MODE			
	);

	-- ----------------------------------------------------- --
	--	CLOCK GENERATOR										 --
	-- ----------------------------------------------------- --
	PROCESS
	BEGIN
		IF( TB_END = '1' )THEN
			WAIT;
		END IF;
		CLK21M <= '0';
		WAIT FOR 5 NS;
		CLK21M <= '1';
		WAIT FOR 5 NS;
	END PROCESS;

	-- ----------------------------------------------------- --
	--	TEST BENCH											 --
	-- ----------------------------------------------------- --
	PROCESS
	BEGIN
		-- INIT
		RESET				<= '1';
		PAL_MODE			<= '0';
		INTERLACE_MODE		<= '0';
		Y212_MODE			<= '1';
		WAIT UNTIL( CLK21M'EVENT AND CLK21M = '1' );
		RESET				<= '0';

		-- SYNCHRONIZE CHANGE MODE -----------------------------

		-- NTSC, NONINTERLACE
		PAL_MODE			<= '0';
		INTERLACE_MODE		<= '0';
		WAIT UNTIL( CLK21M'EVENT AND CLK21M = '1' );

		WAIT UNTIL( FIELD = '1' );
		WAIT UNTIL( FIELD = '0' );
		FOR I IN 0 TO 100 LOOP
			WAIT UNTIL( CLK21M'EVENT AND CLK21M = '1' );
		END LOOP;

		-- PAL, NONINTERLACE
		PAL_MODE			<= '1';
		INTERLACE_MODE		<= '0';
		WAIT UNTIL( CLK21M'EVENT AND CLK21M = '1' );

		WAIT UNTIL( FIELD = '1' );
		WAIT UNTIL( FIELD = '0' );
		FOR I IN 0 TO 10 LOOP
			WAIT UNTIL( CLK21M'EVENT AND CLK21M = '1' );
		END LOOP;

		-- NTSC, INTERLACE
		PAL_MODE			<= '0';
		INTERLACE_MODE		<= '1';
		WAIT UNTIL( CLK21M'EVENT AND CLK21M = '1' );

		WAIT UNTIL( FIELD = '1' );
		WAIT UNTIL( FIELD = '0' );
		FOR I IN 0 TO 10 LOOP
			WAIT UNTIL( CLK21M'EVENT AND CLK21M = '1' );
		END LOOP;

		-- PAL, INTERLACE
		PAL_MODE			<= '1';
		INTERLACE_MODE		<= '1';
		WAIT UNTIL( CLK21M'EVENT AND CLK21M = '1' );

		WAIT UNTIL( FIELD = '1' );
		WAIT UNTIL( FIELD = '0' );
		FOR I IN 0 TO 10 LOOP
			WAIT UNTIL( CLK21M'EVENT AND CLK21M = '1' );
		END LOOP;

		-- ASYNCHRONIZE CHANGE MODE ----------------------------

		-- NTSC, NONINTERLACE
		PAL_MODE			<= '0';
		INTERLACE_MODE		<= '0';
		WAIT UNTIL( CLK21M'EVENT AND CLK21M = '1' );

		WAIT UNTIL( FIELD = '1' );
		FOR I IN 0 TO 100 LOOP
			WAIT UNTIL( CLK21M'EVENT AND CLK21M = '1' );
		END LOOP;

		-- NTSC, INTERLACE (FROM 2ND FIELD IN NTSC-NONINTERLACE)
		PAL_MODE			<= '0';
		INTERLACE_MODE		<= '1';
		WAIT UNTIL( CLK21M'EVENT AND CLK21M = '1' );

		WAIT UNTIL( FIELD = '0' );
		WAIT UNTIL( FIELD = '1' );
		FOR I IN 0 TO 200 LOOP
			WAIT UNTIL( CLK21M'EVENT AND CLK21M = '1' );
		END LOOP;

		-- NTSC, NONINTERLACE (FROM 2ND FIELD IN NTSC-INTERLACE)
		PAL_MODE			<= '0';
		INTERLACE_MODE		<= '0';
		WAIT UNTIL( CLK21M'EVENT AND CLK21M = '1' );

		WAIT UNTIL( FIELD = '0' );
		WAIT UNTIL( FIELD = '1' );
		FOR I IN 0 TO 100 LOOP
			WAIT UNTIL( CLK21M'EVENT AND CLK21M = '1' );
		END LOOP;

		TB_END <= '1';
		WAIT;
	END PROCESS;

END BEHAVIOR;
