-- --------------------------------------------------------- --
--	TEST BENCH												 --
-- ========================================================= --
--	COPYRIGHT (C)2008 T.HARA								 --
-- --------------------------------------------------------- --

LIBRARY	IEEE;
	USE	IEEE.STD_LOGIC_1164.ALL;
	USE	IEEE.STD_LOGIC_UNSIGNED.ALL;
	USE	IEEE.STD_LOGIC_ARITH.ALL;

ENTITY TB IS
END TB;

ARCHITECTURE BEHAVIOR OF TB IS

	-- TEST TARGET
	COMPONENT VDP_WAIT_CONTROL
		PORT(
			RESET			: IN	STD_LOGIC;
			CLK21M			: IN	STD_LOGIC;

			VDP_COMMAND		: IN	STD_LOGIC_VECTOR(  7 DOWNTO 4 );
			HISPEED_MODE	: IN	STD_LOGIC;
			DRIVE			: IN	STD_LOGIC;

			ACTIVE			: OUT	STD_LOGIC
		);
	END COMPONENT;

	CONSTANT CYCLE : TIME := 10 NS;

	SIGNAL RESET				: STD_LOGIC;
	SIGNAL CLK21M				: STD_LOGIC;

	SIGNAL VDP_COMMAND			: STD_LOGIC_VECTOR(  7 DOWNTO 4 );
	SIGNAL HISPEED_MODE			: STD_LOGIC;
	SIGNAL DRIVE				: STD_LOGIC;

	SIGNAL ACTIVE				: STD_LOGIC;

	SIGNAL TB_CLKCNT			: INTEGER := 0;
	SIGNAL TB_CLKCNT_CLR		: STD_LOGIC := '1';
	SIGNAL TB_END				: STD_LOGIC := '0';
BEGIN

	--	INSTANCE
	U_TARGET: VDP_WAIT_CONTROL
	PORT MAP (
		RESET			=> RESET			,
		CLK21M			=> CLK21M			,

		VDP_COMMAND		=> VDP_COMMAND		,
		HISPEED_MODE	=> HISPEED_MODE		,
		DRIVE			=> DRIVE			,

		ACTIVE			=> ACTIVE			
	);

	-- ----------------------------------------------------- --
	--	CLOCK GENERATOR										 --
	-- ----------------------------------------------------- --
	PROCESS
	BEGIN
		IF( TB_END = '1' )THEN
			WAIT;
		END IF;
		CLK21M <= '0';
		WAIT FOR 5 NS;
		CLK21M <= '1';
		WAIT FOR 5 NS;
	END PROCESS;

	PROCESS( CLK21M )
	BEGIN
		IF( CLK21M'EVENT AND CLK21M = '1' )THEN
			IF( TB_CLKCNT_CLR = '1' )THEN
				TB_CLKCNT <= 0;
			ELSE
				TB_CLKCNT <= TB_CLKCNT + 1;
			END IF;
		END IF;
	END PROCESS;

	-- ----------------------------------------------------- --
	--	TEST BENCH											 --
	-- ----------------------------------------------------- --
	PROCESS
	BEGIN
		-- INIT
		RESET			<= '1';
		VDP_COMMAND		<= "1101";	-- HMMM
		HISPEED_MODE	<= '0';
		DRIVE			<= '1';
		WAIT UNTIL( CLK21M'EVENT AND CLK21M = '1' );

		RESET			<= '0';
		FOR I IN 0 TO 100 LOOP
			WAIT UNTIL( CLK21M'EVENT AND CLK21M = '1' );
		END LOOP;

		VDP_COMMAND		<= "0110";	-- SRCH
		FOR I IN 0 TO 100 LOOP
			WAIT UNTIL( CLK21M'EVENT AND CLK21M = '1' );
		END LOOP;

		VDP_COMMAND		<= "0000";	-- STOP
		FOR I IN 0 TO 100 LOOP
			WAIT UNTIL( CLK21M'EVENT AND CLK21M = '1' );
		END LOOP;

		TB_END <= '1';
		WAIT;
	END PROCESS;

END BEHAVIOR;
